@@ -5,11 +5,13 @@ import (
 	"encoding/base32"
 	"encoding/base64"
 	"fmt"
+	"math/rand"
 	"net/http"
 	"sort"
 	"strings"
 	"sync"
 	"syscall"
+	"time"
 	"unicode"
 	"unicode/utf8"
 
@@ -61,6 +63,12 @@ type Bundle struct {
 	res         resolver.Resolver
 	files       []scannerFile
 	entryPoints []graph.EntryPoint
+
+	// The unique key prefix is a random string that is unique to every bundling
+	// operation. It is used as a prefix for the unique keys assigned to every
+	// chunk during linking. These unique keys are used to identify each chunk
+	// before the final output paths have been computed.
+	uniqueKeyPrefix string
 }
 
 type parseArgs struct {
@@ -79,6 +87,7 @@ type parseArgs struct {
 	results         chan parseResult
 	inject          chan config.InjectedFile
 	skipResolve     bool
+	uniqueKeyPrefix string
 }
 
 type parseResult struct {
@@ -131,7 +140,7 @@ func parseFile(args parseArgs) {
 		if !ok {
 			if args.inject != nil {
 				args.inject <- config.InjectedFile{
-					SourceIndex: source.Index,
+					Source: source,
 				}
 			}
 			args.results <- parseResult{}
@@ -236,7 +245,11 @@ func parseFile(args parseArgs) {
 	case config.LoaderBinary:
 		encoded := base64.StdEncoding.EncodeToString([]byte(source.Contents))
 		expr := js_ast.Expr{Data: &js_ast.EString{Value: js_lexer.StringToUTF16(encoded)}}
-		ast := js_parser.LazyExportAST(args.log, source, js_parser.OptionsFromConfig(&args.options), expr, "__toBinary")
+		helper := "__toBinary"
+		if args.options.Platform == config.PlatformNode {
+			helper = "__toBinaryNode"
+		}
+		ast := js_parser.LazyExportAST(args.log, source, js_parser.OptionsFromConfig(&args.options), expr, helper)
 		ast.URLForCSS = "data:application/octet-stream;base64," + encoded
 		if pluginName != "" {
 			result.file.inputFile.SideEffects.Kind = graph.NoSideEffects_PureData_FromPlugin
@@ -262,28 +275,11 @@ func parseFile(args parseArgs) {
 		result.ok = true
 
 	case config.LoaderFile:
-		// Add a hash to the file name to prevent multiple files with the same name
-		// but different contents from colliding
-		var hash string
-		if config.HasPlaceholder(args.options.AssetPathTemplate, config.HashPlaceholder) {
-			h := xxhash.New()
-			h.Write([]byte(source.Contents))
-			hash = hashForFileName(h.Sum(nil))
-		}
-		dir := "/"
-		relPath := config.TemplateToString(config.SubstituteTemplate(args.options.AssetPathTemplate, config.PathPlaceholders{
-			Dir:  &dir,
-			Name: &base,
-			Hash: &hash,
-		})) + ext
-
-		// Determine the final path that this asset will have in the output directory
-		publicPath := joinWithPublicPath(args.options.PublicPath, relPath+source.KeyPath.IgnoredSuffix)
-
-		// Export the resulting relative path as a string
-		expr := js_ast.Expr{Data: &js_ast.EString{Value: js_lexer.StringToUTF16(publicPath)}}
+		uniqueKey := fmt.Sprintf("%sA%08d", args.uniqueKeyPrefix, args.sourceIndex)
+		uniqueKeyPath := uniqueKey + source.KeyPath.IgnoredSuffix
+		expr := js_ast.Expr{Data: &js_ast.EString{Value: js_lexer.StringToUTF16(uniqueKeyPath)}}
 		ast := js_parser.LazyExportAST(args.log, source, js_parser.OptionsFromConfig(&args.options), expr, "")
-		ast.URLForCSS = publicPath
+		ast.URLForCSS = uniqueKeyPath
 		if pluginName != "" {
 			result.file.inputFile.SideEffects.Kind = graph.NoSideEffects_PureData_FromPlugin
 		} else {
@@ -292,27 +288,8 @@ func parseFile(args parseArgs) {
 		result.file.inputFile.Repr = &graph.JSRepr{AST: ast}
 		result.ok = true
 
-		// Optionally add metadata about the file
-		var jsonMetadataChunk string
-		if args.options.NeedsMetafile {
-			inputs := fmt.Sprintf("{\n        %s: {\n          \"bytesInOutput\": %d\n        }\n      }",
-				js_printer.QuoteForJSON(source.PrettyPath, args.options.ASCIIOnly),
-				len(source.Contents),
-			)
-			jsonMetadataChunk = fmt.Sprintf(
-				"{\n      \"imports\": [],\n      \"exports\": [],\n      \"inputs\": %s,\n      \"bytes\": %d\n    }",
-				inputs,
-				len(source.Contents),
-			)
-		}
-
-		// Copy the file using an additional file payload to make sure we only copy
-		// the file if the module isn't removed due to tree shaking.
-		result.file.inputFile.AdditionalFiles = []graph.OutputFile{{
-			AbsPath:           args.fs.Join(args.options.AbsOutputDir, relPath),
-			Contents:          []byte(source.Contents),
-			JSONMetadataChunk: jsonMetadataChunk,
-		}}
+		// Mark that this file is from the "file" loader
+		result.file.inputFile.UniqueKeyForFileLoader = uniqueKey
 
 	default:
 		var message string
@@ -321,23 +298,30 @@ func parseFile(args parseArgs) {
 		} else {
 			message = fmt.Sprintf("Do not know how to load path: %s", source.PrettyPath)
 		}
-		args.log.AddRangeError(args.importSource, args.importPathRange, message)
+		tracker := logger.MakeLineColumnTracker(args.importSource)
+		args.log.AddRangeError(&tracker, args.importPathRange, message)
 	}
 
 	// This must come before we send on the "results" channel to avoid deadlock
 	if args.inject != nil {
-		var exports []string
+		var exports []config.InjectableExport
 		if repr, ok := result.file.inputFile.Repr.(*graph.JSRepr); ok {
-			exports = make([]string, 0, len(repr.AST.NamedExports))
+			aliases := make([]string, 0, len(repr.AST.NamedExports))
 			for alias := range repr.AST.NamedExports {
-				exports = append(exports, alias)
+				aliases = append(aliases, alias)
+			}
+			sort.Strings(aliases) // Sort for determinism
+			exports = make([]config.InjectableExport, len(aliases))
+			for i, alias := range aliases {
+				exports[i] = config.InjectableExport{
+					Alias: alias,
+					Loc:   repr.AST.NamedExports[alias].AliasLoc,
+				}
 			}
-			sort.Strings(exports) // Sort for determinism
 		}
 		args.inject <- config.InjectedFile{
-			Path:        source.PrettyPath,
-			SourceIndex: source.Index,
-			Exports:     exports,
+			Source:  source,
+			Exports: exports,
 		}
 	}
 
@@ -358,6 +342,7 @@ func parseFile(args parseArgs) {
 
 		if len(records) > 0 {
 			resolverCache := make(map[ast.ImportKind]map[string]*resolver.ResolveResult)
+			tracker := logger.MakeLineColumnTracker(&source)
 
 			for importRecordIndex := range records {
 				// Don't try to resolve imports that are already resolved
@@ -404,7 +389,7 @@ func parseFile(args parseArgs) {
 				// want to waste effort traversing into them
 				if record.Kind == ast.ImportRequireResolve {
 					if !record.HandlesImportErrors && (resolveResult == nil || !resolveResult.IsExternal) {
-						args.log.AddRangeWarning(&source, record.Range,
+						args.log.AddRangeWarning(&tracker, record.Range,
 							fmt.Sprintf("%q should be marked as external for use with \"require.resolve\"", record.Path.Text))
 					}
 					continue
@@ -447,7 +432,7 @@ func parseFile(args parseArgs) {
 						}
 						debug.LogErrorMsg(args.log, &source, record.Range, fmt.Sprintf("Could not resolve %q%s", record.Path.Text, hint))
 					} else if args.log.Level <= logger.LevelDebug && !didLogError && record.HandlesImportErrors {
-						args.log.AddRangeDebug(&source, record.Range,
+						args.log.AddRangeDebug(&tracker, record.Range,
 							fmt.Sprintf("Importing %q was allowed even though it could not be resolved because dynamic import failures appear to be handled here",
 								record.Path.Text))
 					}
@@ -533,12 +518,14 @@ func extractSourceMapFromComment(
 	comment js_ast.Span,
 	absResolveDir string,
 ) (logger.Path, *string) {
+	tracker := logger.MakeLineColumnTracker(source)
+
 	// Support data URLs
 	if parsed, ok := resolver.ParseDataURL(comment.Text); ok {
 		if contents, err := parsed.DecodeData(); err == nil {
 			return logger.Path{Text: source.PrettyPath, IgnoredSuffix: "#sourceMappingURL"}, &contents
 		} else {
-			log.AddRangeWarning(source, comment.Range, fmt.Sprintf("Unsupported source map comment: %s", err.Error()))
+			log.AddRangeWarning(&tracker, comment.Range, fmt.Sprintf("Unsupported source map comment: %s", err.Error()))
 			return logger.Path{}, nil
 		}
 	}
@@ -549,21 +536,20 @@ func extractSourceMapFromComment(
 		path := logger.Path{Text: absPath, Namespace: "file"}
 		contents, err, originalError := fsCache.ReadFile(fs, absPath)
 		if log.Level <= logger.LevelDebug && originalError != nil {
-			log.AddRangeDebug(source, comment.Range, fmt.Sprintf("Failed to read file %q: %s", res.PrettyPath(path), originalError.Error()))
+			log.AddRangeDebug(&tracker, comment.Range, fmt.Sprintf("Failed to read file %q: %s", res.PrettyPath(path), originalError.Error()))
 		}
 		if err != nil {
 			if err == syscall.ENOENT {
 				// Don't report a warning because this is likely unactionable
 				return logger.Path{}, nil
 			}
-			log.AddRangeWarning(source, comment.Range, fmt.Sprintf("Cannot read file %q: %s", res.PrettyPath(path), err.Error()))
+			log.AddRangeWarning(&tracker, comment.Range, fmt.Sprintf("Cannot read file %q: %s", res.PrettyPath(path), err.Error()))
 			return logger.Path{}, nil
 		}
 		return path, &contents
 	}
 
 	// Anything else is unsupported
-	log.AddRangeWarning(source, comment.Range, "Unsupported source map comment")
 	return logger.Path{}, nil
 }
 
@@ -588,6 +574,7 @@ func logPluginMessages(
 	importPathRange logger.Range,
 ) bool {
 	didLogError := false
+	tracker := logger.MakeLineColumnTracker(importSource)
 
 	// Report errors and warnings generated by the plugin
 	for _, msg := range msgs {
@@ -603,14 +590,14 @@ func logPluginMessages(
 			sanetizeLocation(res, note.Location)
 		}
 		if msg.Data.Location == nil {
-			msg.Data.Location = logger.LocationOrNil(importSource, importPathRange)
+			msg.Data.Location = logger.LocationOrNil(&tracker, importPathRange)
 		} else {
 			sanetizeLocation(res, msg.Data.Location)
 			if msg.Data.Location.File == "" && importSource != nil {
 				msg.Data.Location.File = importSource.PrettyPath
 			}
 			if importSource != nil {
-				msg.Notes = append(msg.Notes, logger.RangeData(importSource, importPathRange,
+				msg.Notes = append(msg.Notes, logger.RangeData(&tracker, importPathRange,
 					fmt.Sprintf("The plugin %q was triggered by this import", name)))
 			}
 		}
@@ -627,7 +614,7 @@ func logPluginMessages(
 			Kind:       logger.Error,
 			Data: logger.MsgData{
 				Text:       text,
-				Location:   logger.LocationOrNil(importSource, importPathRange),
+				Location:   logger.LocationOrNil(&tracker, importPathRange),
 				UserDetail: thrown,
 			},
 		})
@@ -665,6 +652,7 @@ func runOnResolvePlugins(
 	} else {
 		resolverArgs.Importer.Namespace = importNamespace
 	}
+	tracker := logger.MakeLineColumnTracker(importSource)
 
 	// Apply resolver plugins in order until one succeeds
 	for _, plugin := range plugins {
@@ -713,19 +701,27 @@ func runOnResolvePlugins(
 			// Paths in the file namespace must be absolute paths
 			if result.Path.Namespace == "file" && !fs.IsAbs(result.Path.Text) {
 				if nsFromPlugin == "file" {
-					log.AddRangeError(importSource, importPathRange,
+					log.AddRangeError(&tracker, importPathRange,
 						fmt.Sprintf("Plugin %q returned a path in the \"file\" namespace that is not an absolute path: %s", pluginName, result.Path.Text))
 				} else {
-					log.AddRangeError(importSource, importPathRange,
+					log.AddRangeError(&tracker, importPathRange,
 						fmt.Sprintf("Plugin %q returned a non-absolute path: %s (set a namespace if this is not a file path)", pluginName, result.Path.Text))
 				}
 				return nil, true, resolver.DebugMeta{}
 			}
 
+			var sideEffectsData *resolver.SideEffectsData
+			if result.IsSideEffectFree {
+				sideEffectsData = &resolver.SideEffectsData{
+					PluginName: pluginName,
+				}
+			}
+
 			return &resolver.ResolveResult{
-				PathPair:   resolver.PathPair{Primary: result.Path},
-				IsExternal: result.External,
-				PluginData: result.PluginData,
+				PathPair:               resolver.PathPair{Primary: result.Path},
+				IsExternal:             result.External,
+				PluginData:             result.PluginData,
+				PrimarySideEffectsData: sideEffectsData,
 			}, false, resolver.DebugMeta{}
 		}
 	}
@@ -736,9 +732,9 @@ func runOnResolvePlugins(
 	result, debug := res.Resolve(absResolveDir, path, kind)
 
 	// Warn when the case used for importing differs from the actual file name
-	if result != nil && result.DifferentCase != nil && !resolver.IsInsideNodeModules(absResolveDir) {
+	if result != nil && result.DifferentCase != nil && !helpers.IsInsideNodeModules(absResolveDir) {
 		diffCase := *result.DifferentCase
-		log.AddRangeWarning(importSource, importPathRange, fmt.Sprintf(
+		log.AddRangeWarning(&tracker, importPathRange, fmt.Sprintf(
 			"Use %q instead of %q to avoid issues with case-sensitive file systems",
 			res.PrettyPath(logger.Path{Text: fs.Join(diffCase.Dir, diffCase.Actual), Namespace: "file"}),
 			res.PrettyPath(logger.Path{Text: fs.Join(diffCase.Dir, diffCase.Query), Namespace: "file"}),
@@ -771,6 +767,7 @@ func runOnLoadPlugins(
 		Path:       source.KeyPath,
 		PluginData: pluginData,
 	}
+	tracker := logger.MakeLineColumnTracker(importSource)
 
 	// Apply loader plugins in order until one succeeds
 	for _, plugin := range plugins {
@@ -845,11 +842,11 @@ func runOnLoadPlugins(
 				log.AddDebug(nil, logger.Loc{}, fmt.Sprintf("Failed to read file %q: %s", source.KeyPath.Text, originalError.Error()))
 			}
 			if err == syscall.ENOENT {
-				log.AddRangeError(importSource, importPathRange,
+				log.AddRangeError(&tracker, importPathRange,
 					fmt.Sprintf("Could not read from file: %s", source.KeyPath.Text))
 				return loaderPluginResult{}, false
 			} else {
-				log.AddRangeError(importSource, importPathRange,
+				log.AddRangeError(&tracker, importPathRange,
 					fmt.Sprintf("Cannot read file %q: %s", res.PrettyPath(source.KeyPath), err.Error()))
 				return loaderPluginResult{}, false
 			}
@@ -862,7 +859,7 @@ func runOnLoadPlugins(
 		if parsed, ok := resolver.ParseDataURL(source.KeyPath.Text); ok {
 			if mimeType := parsed.DecodeMIMEType(); mimeType != resolver.MIMETypeUnsupported {
 				if contents, err := parsed.DecodeData(); err != nil {
-					log.AddRangeError(importSource, importPathRange,
+					log.AddRangeError(&tracker, importPathRange,
 						fmt.Sprintf("Could not load data URL: %s", err.Error()))
 					return loaderPluginResult{loader: config.LoaderNone}, true
 				} else {
@@ -901,11 +898,12 @@ func loaderFromFileExtension(extensionToLoader map[string]config.Loader, base st
 	return config.LoaderNone
 }
 
-// Identify the path by its lowercase absolute path name. This should
-// hopefully avoid path case issues on Windows, which has case-insensitive
-// file system paths.
-func lowerCaseAbsPathForWindows(absPath string) string {
-	return strings.ToLower(absPath)
+// Identify the path by its lowercase absolute path name with Windows-specific
+// slashes substituted for standard slashes. This should hopefully avoid path
+// issues on Windows where multiple different paths can refer to the same
+// underlying file.
+func canonicalFileSystemPathForWindows(absPath string) string {
+	return strings.ReplaceAll(strings.ToLower(absPath), "\\", "/")
 }
 
 func hashForFileName(hashBytes []byte) string {
@@ -913,12 +911,13 @@ func hashForFileName(hashBytes []byte) string {
 }
 
 type scanner struct {
-	log     logger.Log
-	fs      fs.FS
-	res     resolver.Resolver
-	caches  *cache.CacheSet
-	options config.Options
-	timer   *helpers.Timer
+	log             logger.Log
+	fs              fs.FS
+	res             resolver.Resolver
+	caches          *cache.CacheSet
+	options         config.Options
+	timer           *helpers.Timer
+	uniqueKeyPrefix string
 
 	// This is not guarded by a mutex because it's only ever modified by a single
 	// thread. Note that not all results in the "results" array are necessarily
@@ -935,6 +934,17 @@ type EntryPoint struct {
 	IsFile     bool
 }
 
+func generateUniqueKeyPrefix() (string, error) {
+	var data [12]byte
+	rand.Seed(time.Now().UnixNano())
+	if _, err := rand.Read(data[:]); err != nil {
+		return "", err
+	}
+
+	// This is 16 bytes and shouldn't generate escape characters when put into strings
+	return base64.URLEncoding.EncodeToString(data[:]), nil
+}
+
 func ScanBundle(
 	log logger.Log,
 	fs fs.FS,
@@ -962,16 +972,23 @@ func ScanBundle(
 		}
 	}
 
+	// Each bundling operation gets a separate unique key
+	uniqueKeyPrefix, err := generateUniqueKeyPrefix()
+	if err != nil {
+		log.AddError(nil, logger.Loc{}, fmt.Sprintf("Failed to read from randomness source: %s", err.Error()))
+	}
+
 	s := scanner{
-		log:           log,
-		fs:            fs,
-		res:           res,
-		caches:        caches,
-		options:       options,
-		timer:         timer,
-		results:       make([]parseResult, 0, caches.SourceIndexCache.LenHint()),
-		visited:       make(map[logger.Path]uint32),
-		resultChannel: make(chan parseResult),
+		log:             log,
+		fs:              fs,
+		res:             res,
+		caches:          caches,
+		options:         options,
+		timer:           timer,
+		results:         make([]parseResult, 0, caches.SourceIndexCache.LenHint()),
+		visited:         make(map[logger.Path]uint32),
+		resultChannel:   make(chan parseResult),
+		uniqueKeyPrefix: uniqueKeyPrefix,
 	}
 
 	// Always start by parsing the runtime file
@@ -997,10 +1014,11 @@ func ScanBundle(
 
 	onStartWaitGroup.Wait()
 	return Bundle{
-		fs:          fs,
-		res:         res,
-		files:       files,
-		entryPoints: entryPointMeta,
+		fs:              fs,
+		res:             res,
+		files:           files,
+		entryPoints:     entryPointMeta,
+		uniqueKeyPrefix: uniqueKeyPrefix,
 	}
 }
 
@@ -1025,7 +1043,7 @@ func (s *scanner) maybeParseFile(
 	path := resolveResult.PathPair.Primary
 	visitedKey := path
 	if visitedKey.Namespace == "file" {
-		visitedKey.Text = lowerCaseAbsPathForWindows(visitedKey.Text)
+		visitedKey.Text = canonicalFileSystemPathForWindows(visitedKey.Text)
 	}
 
 	// Only parse a given file path once
@@ -1042,17 +1060,12 @@ func (s *scanner) maybeParseFile(
 		optionsClone.Stdin = nil
 	}
 
-	// Don't emit warnings for code inside a "node_modules" directory
-	if resolver.IsInsideNodeModules(path.Text) {
-		optionsClone.SuppressWarningsAboutWeirdCode = true
-	}
-
 	// Allow certain properties to be overridden
 	if len(resolveResult.JSXFactory) > 0 {
-		optionsClone.JSX.Factory = resolveResult.JSXFactory
+		optionsClone.JSX.Factory = config.JSXExpr{Parts: resolveResult.JSXFactory}
 	}
 	if len(resolveResult.JSXFragment) > 0 {
-		optionsClone.JSX.Fragment = resolveResult.JSXFragment
+		optionsClone.JSX.Fragment = config.JSXExpr{Parts: resolveResult.JSXFragment}
 	}
 	if resolveResult.UseDefineForClassFieldsTS != config.Unspecified {
 		optionsClone.UseDefineForClassFields = resolveResult.UseDefineForClassFieldsTS
@@ -1060,6 +1073,7 @@ func (s *scanner) maybeParseFile(
 	if resolveResult.PreserveUnusedImportsTS {
 		optionsClone.PreserveUnusedImportsTS = true
 	}
+	optionsClone.TSTarget = resolveResult.TSTarget
 
 	// Set the module type preference using node's module type rules
 	if strings.HasSuffix(path.Text, ".mjs") {
@@ -1114,6 +1128,7 @@ func (s *scanner) maybeParseFile(
 		results:         s.resultChannel,
 		inject:          inject,
 		skipResolve:     skipResolve,
+		uniqueKeyPrefix: s.uniqueKeyPrefix,
 	})
 
 	return sourceIndex
@@ -1165,9 +1180,8 @@ func (s *scanner) preprocessInjectedFiles() {
 		// with the injected defines by index. The index will be used to import
 		// references to them in the parser.
 		injectedFiles = append(injectedFiles, config.InjectedFile{
-			Path:        visitedKey.Text,
-			SourceIndex: sourceIndex,
-			IsDefine:    true,
+			Source:     source,
+			DefineName: define.Name,
 		})
 
 		// Generate the file inline here since it has already been parsed
@@ -1196,14 +1210,14 @@ func (s *scanner) preprocessInjectedFiles() {
 	j := 0
 	for _, absPath := range s.options.InjectAbsPaths {
 		prettyPath := s.res.PrettyPath(logger.Path{Text: absPath, Namespace: "file"})
-		lowerAbsPath := lowerCaseAbsPathForWindows(absPath)
+		absPathKey := canonicalFileSystemPathForWindows(absPath)
 
-		if duplicateInjectedFiles[lowerAbsPath] {
+		if duplicateInjectedFiles[absPathKey] {
 			s.log.AddError(nil, logger.Loc{}, fmt.Sprintf("Duplicate injected file %q", prettyPath))
 			continue
 		}
 
-		duplicateInjectedFiles[lowerAbsPath] = true
+		duplicateInjectedFiles[absPathKey] = true
 		resolveResult := s.res.ResolveAbs(absPath)
 
 		if resolveResult == nil {
@@ -1560,6 +1574,8 @@ func (s *scanner) processScannedFiles() []scannerFile {
 		// Don't try to resolve paths if we're not bundling
 		if s.options.Mode == config.ModeBundle {
 			records := *result.file.inputFile.Repr.ImportRecords()
+			tracker := logger.MakeLineColumnTracker(&result.file.inputFile.Source)
+
 			for importRecordIndex := range records {
 				record := &records[importRecordIndex]
 
@@ -1581,7 +1597,7 @@ func (s *scanner) processScannedFiles() []scannerFile {
 				if resolveResult.PathPair.HasSecondary() {
 					secondaryKey := resolveResult.PathPair.Secondary
 					if secondaryKey.Namespace == "file" {
-						secondaryKey.Text = lowerCaseAbsPathForWindows(secondaryKey.Text)
+						secondaryKey.Text = canonicalFileSystemPathForWindows(secondaryKey.Text)
 					}
 					if secondarySourceIndex, ok := s.visited[secondaryKey]; ok {
 						record.SourceIndex = ast.MakeIndex32(secondarySourceIndex)
@@ -1606,10 +1622,10 @@ func (s *scanner) processScannedFiles() []scannerFile {
 					// Using a JavaScript file with CSS "@import" is not allowed
 					otherFile := &s.results[record.SourceIndex.GetIndex()].file
 					if _, ok := otherFile.inputFile.Repr.(*graph.JSRepr); ok {
-						s.log.AddRangeError(&result.file.inputFile.Source, record.Range,
+						s.log.AddRangeError(&tracker, record.Range,
 							fmt.Sprintf("Cannot import %q into a CSS file", otherFile.inputFile.Source.PrettyPath))
 					} else if record.Kind == ast.ImportAtConditional {
-						s.log.AddRangeError(&result.file.inputFile.Source, record.Range,
+						s.log.AddRangeError(&tracker, record.Range,
 							"Bundling with conditional \"@import\" rules is not currently supported")
 					}
 
@@ -1618,12 +1634,12 @@ func (s *scanner) processScannedFiles() []scannerFile {
 					otherFile := &s.results[record.SourceIndex.GetIndex()].file
 					switch otherRepr := otherFile.inputFile.Repr.(type) {
 					case *graph.CSSRepr:
-						s.log.AddRangeError(&result.file.inputFile.Source, record.Range,
+						s.log.AddRangeError(&tracker, record.Range,
 							fmt.Sprintf("Cannot use %q as a URL", otherFile.inputFile.Source.PrettyPath))
 
 					case *graph.JSRepr:
 						if otherRepr.AST.URLForCSS == "" {
-							s.log.AddRangeError(&result.file.inputFile.Source, record.Range,
+							s.log.AddRangeError(&tracker, record.Range,
 								fmt.Sprintf("Cannot use %q as a URL", otherFile.inputFile.Source.PrettyPath))
 						}
 					}
@@ -1636,12 +1652,12 @@ func (s *scanner) processScannedFiles() []scannerFile {
 					otherFile := &s.results[record.SourceIndex.GetIndex()].file
 					if css, ok := otherFile.inputFile.Repr.(*graph.CSSRepr); ok {
 						if s.options.WriteToStdout {
-							s.log.AddRangeError(&result.file.inputFile.Source, record.Range,
+							s.log.AddRangeError(&tracker, record.Range,
 								fmt.Sprintf("Cannot import %q into a JavaScript file without an output path configured", otherFile.inputFile.Source.PrettyPath))
 						} else if !css.JSSourceIndex.IsValid() {
 							stubKey := otherFile.inputFile.Source.KeyPath
 							if stubKey.Namespace == "file" {
-								stubKey.Text = lowerCaseAbsPathForWindows(stubKey.Text)
+								stubKey.Text = canonicalFileSystemPathForWindows(stubKey.Text)
 							}
 							sourceIndex := s.allocateSourceIndex(stubKey, cache.SourceIndexJSStubForCSS)
 							source := logger.Source{
@@ -1679,25 +1695,32 @@ func (s *scanner) processScannedFiles() []scannerFile {
 				// about it. Note that this can result in esbuild silently generating
 				// broken code. If this actually happens for people, it's probably worth
 				// re-enabling the warning about code inside "node_modules".
-				if record.WasOriginallyBareImport && !s.options.IgnoreDCEAnnotations && !resolver.IsInsideNodeModules(result.file.inputFile.Source.KeyPath.Text) {
+				if record.WasOriginallyBareImport && !s.options.IgnoreDCEAnnotations &&
+					!helpers.IsInsideNodeModules(result.file.inputFile.Source.KeyPath.Text) {
 					if otherModule := &s.results[record.SourceIndex.GetIndex()].file.inputFile; otherModule.SideEffects.Kind != graph.HasSideEffects &&
 						// Do not warn if this is from a plugin, since removing the import
 						// would cause the plugin to not run, and running a plugin is a side
 						// effect.
 						otherModule.SideEffects.Kind != graph.NoSideEffects_PureData_FromPlugin {
 						var notes []logger.MsgData
+						var by string
 						if data := otherModule.SideEffects.Data; data != nil {
-							var text string
-							if data.IsSideEffectsArrayInJSON {
-								text = "It was excluded from the \"sideEffects\" array in the enclosing \"package.json\" file"
+							if data.PluginName != "" {
+								by = fmt.Sprintf(" by plugin %q", data.PluginName)
 							} else {
-								text = "\"sideEffects\" is false in the enclosing \"package.json\" file"
+								var text string
+								if data.IsSideEffectsArrayInJSON {
+									text = "It was excluded from the \"sideEffects\" array in the enclosing \"package.json\" file"
+								} else {
+									text = "\"sideEffects\" is false in the enclosing \"package.json\" file"
+								}
+								tracker := logger.MakeLineColumnTracker(data.Source)
+								notes = append(notes, logger.RangeData(&tracker, data.Range, text))
 							}
-							notes = append(notes, logger.RangeData(data.Source, data.Range, text))
 						}
-						s.log.AddRangeWarningWithNotes(&result.file.inputFile.Source, record.Range,
-							fmt.Sprintf("Ignoring this import because %q was marked as having no side effects",
-								otherModule.Source.PrettyPath), notes)
+						s.log.AddRangeWarningWithNotes(&tracker, record.Range,
+							fmt.Sprintf("Ignoring this import because %q was marked as having no side effects%s",
+								otherModule.Source.PrettyPath, by), notes)
 					}
 				}
 			}
@@ -1711,7 +1734,62 @@ func (s *scanner) processScannedFiles() []scannerFile {
 			sb.WriteString("]\n    }")
 		}
 
-		s.results[i].file.jsonMetadataChunk = sb.String()
+		result.file.jsonMetadataChunk = sb.String()
+
+		// If this file is from the "file" loader, generate an additional file
+		if result.file.inputFile.UniqueKeyForFileLoader != "" {
+			bytes := []byte(result.file.inputFile.Source.Contents)
+
+			// Add a hash to the file name to prevent multiple files with the same name
+			// but different contents from colliding
+			var hash string
+			if config.HasPlaceholder(s.options.AssetPathTemplate, config.HashPlaceholder) {
+				h := xxhash.New()
+				h.Write(bytes)
+				hash = hashForFileName(h.Sum(nil))
+			}
+
+			// Generate the input for the template
+			_, _, originalExt := logger.PlatformIndependentPathDirBaseExt(result.file.inputFile.Source.KeyPath.Text)
+			dir, base, ext := pathRelativeToOutbase(
+				&result.file.inputFile,
+				&s.options,
+				s.fs,
+				originalExt,
+				/* avoidIndex */ false,
+				/* customFilePath */ "",
+			)
+
+			// Apply the asset path template
+			relPath := config.TemplateToString(config.SubstituteTemplate(s.options.AssetPathTemplate, config.PathPlaceholders{
+				Dir:  &dir,
+				Name: &base,
+				Hash: &hash,
+			})) + ext
+
+			// Optionally add metadata about the file
+			var jsonMetadataChunk string
+			if s.options.NeedsMetafile {
+				inputs := fmt.Sprintf("{\n        %s: {\n          \"bytesInOutput\": %d\n        }\n      }",
+					js_printer.QuoteForJSON(result.file.inputFile.Source.PrettyPath, s.options.ASCIIOnly),
+					len(bytes),
+				)
+				jsonMetadataChunk = fmt.Sprintf(
+					"{\n      \"imports\": [],\n      \"exports\": [],\n      \"inputs\": %s,\n      \"bytes\": %d\n    }",
+					inputs,
+					len(bytes),
+				)
+			}
+
+			// Generate the additional file to copy into the output directory
+			result.file.inputFile.AdditionalFiles = []graph.OutputFile{{
+				AbsPath:           s.fs.Join(s.options.AbsOutputDir, relPath),
+				Contents:          bytes,
+				JSONMetadataChunk: jsonMetadataChunk,
+			}}
+		}
+
+		s.results[i] = result
 	}
 
 	// The linker operates on an array of files, so construct that now. This
@@ -1765,7 +1843,8 @@ func (s *scanner) validateTLA(sourceIndex uint32) tlaCheck {
 
 							if parentRepr.AST.TopLevelAwaitKeyword.Len > 0 {
 								tlaPrettyPath = parentResult.file.inputFile.Source.PrettyPath
-								notes = append(notes, logger.RangeData(&parentResult.file.inputFile.Source, parentRepr.AST.TopLevelAwaitKeyword,
+								tracker := logger.MakeLineColumnTracker(&parentResult.file.inputFile.Source)
+								notes = append(notes, logger.RangeData(&tracker, parentRepr.AST.TopLevelAwaitKeyword,
 									fmt.Sprintf("The top-level await in %q is here", tlaPrettyPath)))
 								break
 							}
@@ -1777,7 +1856,8 @@ func (s *scanner) validateTLA(sourceIndex uint32) tlaCheck {
 
 							otherSourceIndex = parentResult.tlaCheck.parent.GetIndex()
 
-							notes = append(notes, logger.RangeData(&parentResult.file.inputFile.Source,
+							tracker := logger.MakeLineColumnTracker(&parentResult.file.inputFile.Source)
+							notes = append(notes, logger.RangeData(&tracker,
 								parentRepr.AST.ImportRecords[parent.importRecordIndex].Range,
 								fmt.Sprintf("The file %q imports the file %q here",
 									parentResult.file.inputFile.Source.PrettyPath, s.results[otherSourceIndex].file.inputFile.Source.PrettyPath)))
@@ -1794,7 +1874,8 @@ func (s *scanner) validateTLA(sourceIndex uint32) tlaCheck {
 								tlaPrettyPath)
 						}
 
-						s.log.AddRangeErrorWithNotes(&result.file.inputFile.Source, record.Range, text, notes)
+						tracker := logger.MakeLineColumnTracker(&result.file.inputFile.Source)
+						s.log.AddRangeErrorWithNotes(&tracker, record.Range, text, notes)
 					}
 				}
 			}
@@ -1855,6 +1936,8 @@ func applyOptionDefaults(options *config.Options) {
 			{Data: "-", Placeholder: config.HashPlaceholder},
 		}
 	}
+
+	options.ProfilerNames = !options.MinifyIdentifiers
 }
 
 func (b *Bundle) Compile(log logger.Log, options config.Options, timer *helpers.Timer) ([]graph.OutputFile, string) {
@@ -1884,7 +1967,8 @@ func (b *Bundle) Compile(log logger.Log, options config.Options, timer *helpers.
 	var resultGroups [][]graph.OutputFile
 	if options.CodeSplitting || len(b.entryPoints) == 1 {
 		// If code splitting is enabled or if there's only one entry point, link all entry points together
-		resultGroups = [][]graph.OutputFile{link(&options, timer, log, b.fs, b.res, files, b.entryPoints, allReachableFiles, dataForSourceMaps)}
+		resultGroups = [][]graph.OutputFile{link(
+			&options, timer, log, b.fs, b.res, files, b.entryPoints, b.uniqueKeyPrefix, allReachableFiles, dataForSourceMaps)}
 	} else {
 		// Otherwise, link each entry point with the runtime file separately
 		waitGroup := sync.WaitGroup{}
@@ -1895,7 +1979,8 @@ func (b *Bundle) Compile(log logger.Log, options config.Options, timer *helpers.
 				entryPoints := []graph.EntryPoint{entryPoint}
 				forked := timer.Fork()
 				reachableFiles := findReachableFiles(files, entryPoints)
-				resultGroups[i] = link(&options, forked, log, b.fs, b.res, files, entryPoints, reachableFiles, dataForSourceMaps)
+				resultGroups[i] = link(
+					&options, forked, log, b.fs, b.res, files, entryPoints, b.uniqueKeyPrefix, reachableFiles, dataForSourceMaps)
 				timer.Join(forked)
 				waitGroup.Done()
 			}(i, entryPoint)
@@ -1924,13 +2009,13 @@ func (b *Bundle) Compile(log logger.Log, options config.Options, timer *helpers.
 			for _, sourceIndex := range allReachableFiles {
 				keyPath := b.files[sourceIndex].inputFile.Source.KeyPath
 				if keyPath.Namespace == "file" {
-					lowerAbsPath := lowerCaseAbsPathForWindows(keyPath.Text)
-					sourceAbsPaths[lowerAbsPath] = sourceIndex
+					absPathKey := canonicalFileSystemPathForWindows(keyPath.Text)
+					sourceAbsPaths[absPathKey] = sourceIndex
 				}
 			}
 			for _, outputFile := range outputFiles {
-				lowerAbsPath := lowerCaseAbsPathForWindows(outputFile.AbsPath)
-				if sourceIndex, ok := sourceAbsPaths[lowerAbsPath]; ok {
+				absPathKey := canonicalFileSystemPathForWindows(outputFile.AbsPath)
+				if sourceIndex, ok := sourceAbsPaths[absPathKey]; ok {
 					hint := ""
 					switch logger.API {
 					case logger.CLIAPI:
@@ -1956,12 +2041,12 @@ func (b *Bundle) Compile(log logger.Log, options config.Options, timer *helpers.
 		outputFileMap := make(map[string][]byte)
 		end := 0
 		for _, outputFile := range outputFiles {
-			lowerAbsPath := lowerCaseAbsPathForWindows(outputFile.AbsPath)
-			contents, ok := outputFileMap[lowerAbsPath]
+			absPathKey := canonicalFileSystemPathForWindows(outputFile.AbsPath)
+			contents, ok := outputFileMap[absPathKey]
 
 			// If this isn't a duplicate, keep the output file
 			if !ok {
-				outputFileMap[lowerAbsPath] = outputFile.Contents
+				outputFileMap[absPathKey] = outputFile.Contents
 				outputFiles[end] = outputFile
 				end++
 				continue
@@ -2143,15 +2228,11 @@ type runtimeCacheKey struct {
 	MangleSyntax      bool
 	MinifyIdentifiers bool
 	ES6               bool
-	Platform          config.Platform
 }
 
 type runtimeCache struct {
 	astMutex sync.Mutex
 	astMap   map[runtimeCacheKey]js_ast.AST
-
-	definesMutex sync.Mutex
-	definesMap   map[config.Platform]*config.ProcessedDefines
 }
 
 var globalRuntimeCache runtimeCache
@@ -2161,7 +2242,6 @@ func (cache *runtimeCache) parseRuntime(options *config.Options) (source logger.
 		// All configuration options that the runtime code depends on must go here
 		MangleSyntax:      options.MangleSyntax,
 		MinifyIdentifiers: options.MinifyIdentifiers,
-		Platform:          options.Platform,
 		ES6:               runtime.CanUseES6(options.UnsupportedJSFeatures),
 	}
 
@@ -2191,13 +2271,11 @@ func (cache *runtimeCache) parseRuntime(options *config.Options) (source logger.
 	} else {
 		constraint = 5
 	}
-	log := logger.NewDeferLog()
+	log := logger.NewDeferLog(logger.DeferLogAll)
 	runtimeAST, ok = js_parser.Parse(log, source, js_parser.OptionsFromConfig(&config.Options{
 		// These configuration options must only depend on the key
 		MangleSyntax:      key.MangleSyntax,
 		MinifyIdentifiers: key.MinifyIdentifiers,
-		Platform:          key.Platform,
-		Defines:           cache.processedDefines(key.Platform),
 		UnsupportedJSFeatures: compat.UnsupportedJSFeatures(
 			map[compat.Engine][]int{compat.ES: {constraint}}),
 
@@ -2224,47 +2302,3 @@ func (cache *runtimeCache) parseRuntime(options *config.Options) (source logger.
 	}
 	return
 }
-
-func (cache *runtimeCache) processedDefines(key config.Platform) (defines *config.ProcessedDefines) {
-	ok := false
-
-	// Cache hit?
-	(func() {
-		cache.definesMutex.Lock()
-		defer cache.definesMutex.Unlock()
-		if cache.definesMap != nil {
-			defines, ok = cache.definesMap[key]
-		}
-	})()
-	if ok {
-		return
-	}
-
-	// Cache miss
-	var platform string
-	switch key {
-	case config.PlatformBrowser:
-		platform = "browser"
-	case config.PlatformNode:
-		platform = "node"
-	case config.PlatformNeutral:
-		platform = "neutral"
-	}
-	result := config.ProcessDefines(map[string]config.DefineData{
-		"__platform": {
-			DefineFunc: func(config.DefineArgs) js_ast.E {
-				return &js_ast.EString{Value: js_lexer.StringToUTF16(platform)}
-			},
-		},
-	})
-	defines = &result
-
-	// Cache for next time
-	cache.definesMutex.Lock()
-	defer cache.definesMutex.Unlock()
-	if cache.definesMap == nil {
-		cache.definesMap = make(map[config.Platform]*config.ProcessedDefines)
-	}
-	cache.definesMap[key] = defines
-	return
-}
