@@ -863,8 +863,12 @@ func TestLowerAsyncThis2016ES6(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
+				export {bar} from "./other"
 				export let foo = async () => this
 			`,
+			"/other.js": `
+				export let bar = async () => {}
+			`,
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
@@ -872,6 +876,9 @@ func TestLowerAsyncThis2016ES6(t *testing.T) {
 			UnsupportedJSFeatures: es(2016),
 			AbsOutputFile:         "/out.js",
 		},
+		expectedScanLog: `entry.js: warning: Top-level "this" will be replaced with undefined since this file is an ECMAScript module
+entry.js: note: This file is considered an ECMAScript module because of the "export" keyword here
+`,
 	})
 }
 
@@ -1537,3 +1544,52 @@ func TestLowerPrivateClassBrandCheckSupported(t *testing.T) {
 		},
 	})
 }
+
+func TestLowerTemplateObject(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				x = () => [
+					tag` + "`x`" + `,
+					tag` + "`\\xFF`" + `,
+					tag` + "`\\x`" + `,
+					tag` + "`\\u`" + `,
+				]
+				y = () => [
+					tag` + "`x${y}z`" + `,
+					tag` + "`\\xFF${y}z`" + `,
+					tag` + "`x${y}\\z`" + `,
+					tag` + "`x${y}\\u`" + `,
+				]
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:                  config.ModePassThrough,
+			AbsOutputFile:         "/out.js",
+			UnsupportedJSFeatures: compat.TemplateLiteral,
+		},
+	})
+}
+
+// See https://github.com/evanw/esbuild/issues/1424 for more information
+func TestLowerPrivateClassFieldStaticIssue1424(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				class T {
+					#a() { return 'a'; }
+					#b() { return 'b'; }
+					static c;
+					d() { console.log(this.#a()); }
+				}
+				new T().d();
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
