@@ -3,6 +3,7 @@ package bundler
 import (
 	"testing"
 
+	"github.com/evanw/esbuild/internal/compat"
 	"github.com/evanw/esbuild/internal/config"
 )
 
@@ -90,6 +91,48 @@ func TestTSDeclareClass(t *testing.T) {
 	})
 }
 
+func TestTSDeclareClassFields(t *testing.T) {
+	// Note: this test uses arrow functions to validate that
+	// scopes inside "declare" fields are correctly discarded
+	ts_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.ts": `
+				import './define-false'
+				import './define-true'
+			`,
+			"/define-false/index.ts": `
+				class Foo {
+					a
+					declare b
+					c = () => this
+					declare d = () => this
+				}
+				(() => new Foo())()
+			`,
+			"/define-true/index.ts": `
+				class Bar {
+					a
+					declare b
+					c = () => this
+					declare d = () => this
+				}
+				(() => new Bar())()
+			`,
+			"/define-true/tsconfig.json": `{
+				"compilerOptions": {
+					"useDefineForClassFields": true
+				}
+			}`,
+		},
+		entryPaths: []string{"/entry.ts"},
+		options: config.Options{
+			Mode:                  config.ModeBundle,
+			AbsOutputFile:         "/out.js",
+			UnsupportedJSFeatures: compat.ClassField,
+		},
+	})
+}
+
 func TestTSDeclareFunction(t *testing.T) {
 	ts_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -453,6 +496,29 @@ func TestTSImportVsLocalCollisionMixed(t *testing.T) {
 	})
 }
 
+func TestTSImportEqualsEliminationTest(t *testing.T) {
+	ts_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.ts": `
+				import a = foo.a
+				import b = a.b
+				import c = b.c
+
+				import x = foo.x
+				import y = x.y
+				import z = y.z
+
+				export let bar = c
+			`,
+		},
+		entryPaths: []string{"/entry.ts"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
 func TestTSMinifiedBundleES6(t *testing.T) {
 	ts_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -645,6 +711,23 @@ func TestTypeScriptDecorators(t *testing.T) {
 	})
 }
 
+func TestTypeScriptDecoratorsKeepNames(t *testing.T) {
+	ts_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.ts": `
+				@decoratorMustComeAfterName
+				class Foo {}
+			`,
+		},
+		entryPaths: []string{"/entry.ts"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			KeepNames:     true,
+		},
+	})
+}
+
 func TestTSExportDefaultTypeIssue316(t *testing.T) {
 	ts_suite.expectBundled(t, bundled{
 		files: map[string]string{
