@@ -14,7 +14,6 @@ import (
 type T uint8
 
 const eof = -1
-const replacementCharacter = 0xFFFD
 
 const (
 	TEndOfFile T = iota
@@ -101,6 +100,14 @@ func (t T) String() string {
 	return tokenToString[t]
 }
 
+func (t T) IsNumeric() bool {
+	return t == TNumber || t == TPercentage || t == TDimension
+}
+
+func (t T) IsNumericOrIdent() bool {
+	return t.IsNumeric() || t == TIdent
+}
+
 // This token struct is designed to be memory-efficient. It just references a
 // range in the input file instead of directly containing the substring of text
 // since a range takes up less memory than a string.
@@ -148,6 +155,7 @@ func (token Token) DecodedText(contents string) string {
 type lexer struct {
 	log       logger.Log
 	source    logger.Source
+	tracker   logger.LineColumnTracker
 	current   int
 	codePoint rune
 	Token     Token
@@ -155,8 +163,9 @@ type lexer struct {
 
 func Tokenize(log logger.Log, source logger.Source) (tokens []Token) {
 	lexer := lexer{
-		log:    log,
-		source: source,
+		log:     log,
+		source:  source,
+		tracker: logger.MakeLineColumnTracker(&source),
 	}
 	lexer.step()
 
@@ -339,7 +348,7 @@ func (lexer *lexer) next() {
 				lexer.Token.Kind = lexer.consumeIdentLike()
 			} else {
 				lexer.step()
-				lexer.log.AddRangeError(&lexer.source, lexer.Token.Range, "Invalid escape")
+				lexer.log.AddRangeError(&lexer.tracker, lexer.Token.Range, "Invalid escape")
 				lexer.Token.Kind = TDelim
 			}
 
@@ -406,8 +415,8 @@ func (lexer *lexer) consumeToEndOfMultiLineComment(startRange logger.Range) {
 			}
 
 		case eof: // This indicates the end of the file
-			lexer.log.AddErrorWithNotes(&lexer.source, logger.Loc{Start: lexer.Token.Range.End()}, "Expected \"*/\" to terminate multi-line comment",
-				[]logger.MsgData{logger.RangeData(&lexer.source, startRange, "The multi-line comment starts here")})
+			lexer.log.AddErrorWithNotes(&lexer.tracker, logger.Loc{Start: lexer.Token.Range.End()}, "Expected \"*/\" to terminate multi-line comment",
+				[]logger.MsgData{logger.RangeData(&lexer.tracker, startRange, "The multi-line comment starts here")})
 			return
 
 		default:
@@ -420,7 +429,7 @@ func (lexer *lexer) consumeToEndOfSingleLineComment() {
 	for !isNewline(lexer.codePoint) && lexer.codePoint != eof {
 		lexer.step()
 	}
-	lexer.log.AddRangeWarning(&lexer.source, lexer.Token.Range, "Comments in CSS use \"/* ... */\" instead of \"//\"")
+	lexer.log.AddRangeWarning(&lexer.tracker, lexer.Token.Range, "Comments in CSS use \"/* ... */\" instead of \"//\"")
 }
 
 func (lexer *lexer) isValidEscape() bool {
@@ -437,13 +446,16 @@ func (lexer *lexer) wouldStartIdentifier() bool {
 	}
 
 	if lexer.codePoint == '-' {
-		c, w := utf8.DecodeRuneInString(lexer.source.Contents[lexer.current:])
+		c, width := utf8.DecodeRuneInString(lexer.source.Contents[lexer.current:])
+		if c == utf8.RuneError && width <= 1 {
+			return false // Decoding error
+		}
 		if IsNameStart(c) || c == '-' {
 			return true
 		}
 		if c == '\\' {
-			c, _ = utf8.DecodeRuneInString(lexer.source.Contents[lexer.current+w:])
-			return !isNewline(c)
+			c2, _ := utf8.DecodeRuneInString(lexer.source.Contents[lexer.current+width:])
+			return !isNewline(c2)
 		}
 		return false
 	}
@@ -452,14 +464,21 @@ func (lexer *lexer) wouldStartIdentifier() bool {
 }
 
 func WouldStartIdentifierWithoutEscapes(text string) bool {
-	if len(text) > 0 {
-		c, width := utf8.DecodeRuneInString(text)
-		if IsNameStart(c) {
+	c, width := utf8.DecodeRuneInString(text)
+	if c == utf8.RuneError && width <= 1 {
+		return false // Decoding error
+	}
+	if IsNameStart(c) {
+		return true
+	}
+
+	if c == '-' {
+		c2, width2 := utf8.DecodeRuneInString(text[width:])
+		if c2 == utf8.RuneError && width2 <= 1 {
+			return false // Decoding error
+		}
+		if IsNameStart(c2) || c2 == '-' {
 			return true
-		} else if c == '-' {
-			if c, _ := utf8.DecodeRuneInString(text[width:]); IsNameStart(c) || c == '-' {
-				return true
-			}
 		}
 	}
 	return false
@@ -536,13 +555,13 @@ func (lexer *lexer) consumeEscape() rune {
 			lexer.step()
 		}
 		if hex == 0 || (hex >= 0xD800 && hex <= 0xDFFF) || hex > 0x10FFFF {
-			return replacementCharacter
+			return utf8.RuneError
 		}
 		return rune(hex)
 	}
 
 	if c == eof {
-		return replacementCharacter
+		return utf8.RuneError
 	}
 
 	lexer.step()
@@ -581,7 +600,7 @@ validURL:
 
 		case eof:
 			loc := logger.Loc{Start: lexer.Token.Range.End()}
-			lexer.log.AddError(&lexer.source, loc, "Expected \")\" to end URL token")
+			lexer.log.AddError(&lexer.tracker, loc, "Expected \")\" to end URL token")
 			return TBadURL
 
 		case ' ', '\t', '\n', '\r', '\f':
@@ -591,7 +610,7 @@ validURL:
 			}
 			if lexer.codePoint != ')' {
 				loc := logger.Loc{Start: lexer.Token.Range.End()}
-				lexer.log.AddError(&lexer.source, loc, "Expected \")\" to end URL token")
+				lexer.log.AddError(&lexer.tracker, loc, "Expected \")\" to end URL token")
 				break validURL
 			}
 			lexer.step()
@@ -599,13 +618,13 @@ validURL:
 
 		case '"', '\'', '(':
 			r := logger.Range{Loc: logger.Loc{Start: lexer.Token.Range.End()}, Len: 1}
-			lexer.log.AddRangeError(&lexer.source, r, "Expected \")\" to end URL token")
+			lexer.log.AddRangeError(&lexer.tracker, r, "Expected \")\" to end URL token")
 			break validURL
 
 		case '\\':
 			if !lexer.isValidEscape() {
 				r := logger.Range{Loc: logger.Loc{Start: lexer.Token.Range.End()}, Len: 1}
-				lexer.log.AddRangeError(&lexer.source, r, "Invalid escape")
+				lexer.log.AddRangeError(&lexer.tracker, r, "Invalid escape")
 				break validURL
 			}
 			lexer.consumeEscape()
@@ -613,7 +632,7 @@ validURL:
 		default:
 			if isNonPrintable(lexer.codePoint) {
 				r := logger.Range{Loc: logger.Loc{Start: lexer.Token.Range.End()}, Len: 1}
-				lexer.log.AddRangeError(&lexer.source, r, "Unexpected non-printable character in URL token")
+				lexer.log.AddRangeError(&lexer.tracker, r, "Unexpected non-printable character in URL token")
 			}
 			lexer.step()
 		}
@@ -656,11 +675,11 @@ func (lexer *lexer) consumeString() T {
 			// Otherwise, fall through to ignore the character after the backslash
 
 		case eof:
-			lexer.log.AddError(&lexer.source, logger.Loc{Start: lexer.Token.Range.End()}, "Unterminated string token")
+			lexer.log.AddError(&lexer.tracker, logger.Loc{Start: lexer.Token.Range.End()}, "Unterminated string token")
 			return TBadString
 
 		case '\n', '\r', '\f':
-			lexer.log.AddError(&lexer.source, logger.Loc{Start: lexer.Token.Range.End()}, "Unterminated string token")
+			lexer.log.AddError(&lexer.tracker, logger.Loc{Start: lexer.Token.Range.End()}, "Unterminated string token")
 			return TBadString
 
 		case quote:
@@ -728,7 +747,7 @@ func (lexer *lexer) consumeNumeric() T {
 }
 
 func IsNameStart(c rune) bool {
-	return (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') || c == '_' || c >= 0x80
+	return (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') || c == '_' || c >= 0x80 || c == '\x00'
 }
 
 func IsNameContinue(c rune) bool {
@@ -772,7 +791,7 @@ func decodeEscapesInToken(inner string) string {
 	i := 0
 
 	for i < len(inner) {
-		if inner[i] == '\\' {
+		if c := inner[i]; c == '\\' || c == '\x00' {
 			break
 		}
 		i++
@@ -791,12 +810,15 @@ func decodeEscapesInToken(inner string) string {
 		inner = inner[width:]
 
 		if c != '\\' {
+			if c == '\x00' {
+				c = utf8.RuneError
+			}
 			sb.WriteRune(c)
 			continue
 		}
 
 		if len(inner) == 0 {
-			sb.WriteRune(replacementCharacter)
+			sb.WriteRune(utf8.RuneError)
 			continue
 		}
 
@@ -843,7 +865,7 @@ func decodeEscapesInToken(inner string) string {
 		}
 
 		if hex == 0 || (hex >= 0xD800 && hex <= 0xDFFF) || hex > 0x10FFFF {
-			sb.WriteRune(replacementCharacter)
+			sb.WriteRune(utf8.RuneError)
 			continue
 		}
 
