@@ -1,9 +1,11 @@
 package renamer
 
 import (
+	"fmt"
 	"sort"
 	"strconv"
 	"sync"
+	"sync/atomic"
 
 	"github.com/evanw/esbuild/internal/ast"
 	"github.com/evanw/esbuild/internal/js_ast"
@@ -66,16 +68,16 @@ func (r *noOpRenamer) NameForSymbol(ref js_ast.Ref) string {
 // MinifyRenamer
 
 type symbolSlot struct {
-	name  string
-	count uint32
+	name               string
+	count              uint32
+	needsCapitalForJSX uint32 // This is really a bool but needs to be atomic
 }
 
 type MinifyRenamer struct {
-	symbols       js_ast.SymbolMap
-	sortedBuffer  StableRefArray
-	reservedNames map[string]uint32
-	slots         [3][]symbolSlot
-	symbolToSlot  map[js_ast.Ref]ast.Index32
+	symbols              js_ast.SymbolMap
+	reservedNames        map[string]uint32
+	slots                [3][]symbolSlot
+	topLevelSymbolToSlot map[js_ast.Ref]uint32
 }
 
 func NewMinifyRenamer(symbols js_ast.SymbolMap, firstTopLevelSlots js_ast.SlotCounts, reservedNames map[string]uint32) *MinifyRenamer {
@@ -87,7 +89,7 @@ func NewMinifyRenamer(symbols js_ast.SymbolMap, firstTopLevelSlots js_ast.SlotCo
 			make([]symbolSlot, firstTopLevelSlots[1]),
 			make([]symbolSlot, firstTopLevelSlots[2]),
 		},
-		symbolToSlot: make(map[js_ast.Ref]ast.Index32),
+		topLevelSymbolToSlot: make(map[js_ast.Ref]uint32),
 	}
 }
 
@@ -107,8 +109,7 @@ func (r *MinifyRenamer) NameForSymbol(ref js_ast.Ref) string {
 
 	// If it's not (i.e. it's in a top-level scope), look up the slot
 	if !i.IsValid() {
-		var ok bool
-		i, ok = r.symbolToSlot[ref]
+		index, ok := r.topLevelSymbolToSlot[ref]
 		if !ok {
 			// If we get here, then we're printing a symbol that never had any
 			// recorded uses. This is odd but can happen in certain scenarios.
@@ -117,29 +118,67 @@ func (r *MinifyRenamer) NameForSymbol(ref js_ast.Ref) string {
 			// what name we use since it's dead code.
 			return symbol.OriginalName
 		}
+		i = ast.MakeIndex32(index)
 	}
 
 	return r.slots[ns][i.GetIndex()].name
 }
 
-func (r *MinifyRenamer) AccumulateSymbolUseCounts(symbolUses map[js_ast.Ref]js_ast.SymbolUse, stableSourceIndices []uint32) {
-	// Sort symbol uses for determinism, reusing a shared memory buffer
-	r.sortedBuffer = r.sortedBuffer[:0]
-	for ref := range symbolUses {
-		r.sortedBuffer = append(r.sortedBuffer, StableRef{
-			StableSourceIndex: stableSourceIndices[ref.SourceIndex],
-			Ref:               ref,
-		})
-	}
-	sort.Sort(r.sortedBuffer)
+// The sort order here is arbitrary but needs to be consistent between builds.
+// The InnerIndex should be stable because the parser for a single file is
+// single-threaded and deterministically assigns out InnerIndex values
+// sequentially. But the SourceIndex should be unstable because the main thread
+// assigns out source index values sequentially to newly-discovered dependencies
+// in a multi-threaded producer/consumer relationship. So instead we use the
+// index of the source in the DFS order over all entry points for stability.
+type DeferredTopLevelSymbol struct {
+	StableSourceIndex uint32
+	Ref               js_ast.Ref
+	Count             uint32
+}
 
-	// Accumulate symbol use counts
-	for _, stable := range r.sortedBuffer {
-		r.AccumulateSymbolCount(stable.Ref, symbolUses[stable.Ref].CountEstimate)
+// This type is just so we can use Go's native sort function
+type DeferredTopLevelSymbolArray []DeferredTopLevelSymbol
+
+func (a DeferredTopLevelSymbolArray) Len() int          { return len(a) }
+func (a DeferredTopLevelSymbolArray) Swap(i int, j int) { a[i], a[j] = a[j], a[i] }
+func (a DeferredTopLevelSymbolArray) Less(i int, j int) bool {
+	ai, aj := a[i], a[j]
+	if ai.StableSourceIndex < aj.StableSourceIndex {
+		return true
+	}
+	if ai.StableSourceIndex > aj.StableSourceIndex {
+		return false
+	}
+	if ai.Ref.InnerIndex < aj.Ref.InnerIndex {
+		return true
+	}
+	if ai.Ref.InnerIndex > aj.Ref.InnerIndex {
+		return false
+	}
+	return ai.Count < aj.Count
+}
+
+func (r *MinifyRenamer) AccumulateSymbolUseCounts(
+	topLevelSymbols *DeferredTopLevelSymbolArray,
+	symbolUses map[js_ast.Ref]js_ast.SymbolUse,
+	stableSourceIndices []uint32,
+) {
+	// NOTE: This function is run in parallel. Make sure to avoid data races.
+
+	for ref, use := range symbolUses {
+		r.AccumulateSymbolCount(topLevelSymbols, ref, use.CountEstimate, stableSourceIndices)
 	}
 }
 
-func (r *MinifyRenamer) AccumulateSymbolCount(ref js_ast.Ref, count uint32) {
+func (r *MinifyRenamer) AccumulateSymbolCount(
+	topLevelSymbols *DeferredTopLevelSymbolArray,
+	ref js_ast.Ref,
+	count uint32,
+	stableSourceIndices []uint32,
+) {
+	// NOTE: This function is run in parallel. Make sure to avoid data races.
+
 	// Follow links to get to the underlying symbol
 	ref = js_ast.FollowSymbols(r.symbols, ref)
 	symbol := r.symbols.Get(ref)
@@ -155,21 +194,52 @@ func (r *MinifyRenamer) AccumulateSymbolCount(ref js_ast.Ref, count uint32) {
 	}
 
 	// Check if it's a nested scope symbol
-	slots := &r.slots[ns]
-	i := symbol.NestedScopeSlot
-
-	// If it's not (i.e. it's in a top-level scope), allocate a slot for it
-	if !i.IsValid() {
-		var ok bool
-		i, ok = r.symbolToSlot[ref]
-		if !ok {
-			i = ast.MakeIndex32(uint32(len(*slots)))
-			*slots = append(*slots, symbolSlot{})
-			r.symbolToSlot[ref] = i
+	if i := symbol.NestedScopeSlot; i.IsValid() {
+		// If it is, accumulate the count using a parallel-safe atomic increment
+		slot := &r.slots[ns][i.GetIndex()]
+		atomic.AddUint32(&slot.count, count)
+		if symbol.MustStartWithCapitalLetterForJSX {
+			atomic.StoreUint32(&slot.needsCapitalForJSX, 1)
 		}
+		return
 	}
 
-	(*slots)[i.GetIndex()].count += count
+	// If it's a top-level symbol, defer it to later since we have
+	// to allocate slots for these in serial instead of in parallel
+	*topLevelSymbols = append(*topLevelSymbols, DeferredTopLevelSymbol{
+		StableSourceIndex: stableSourceIndices[ref.SourceIndex],
+		Ref:               ref,
+		Count:             count,
+	})
+}
+
+// The parallel part of the symbol count accumulation algorithm above processes
+// nested symbols and generates on an array of top-level symbols to process later.
+// After the parallel part has finished, that array of top-level symbols is passed
+// to this function which processes them in serial.
+func (r *MinifyRenamer) AllocateTopLevelSymbolSlots(topLevelSymbols DeferredTopLevelSymbolArray) {
+	for _, stable := range topLevelSymbols {
+		symbol := r.symbols.Get(stable.Ref)
+		slots := &r.slots[symbol.SlotNamespace()]
+		if i, ok := r.topLevelSymbolToSlot[stable.Ref]; ok {
+			slot := &(*slots)[i]
+			slot.count += stable.Count
+			if symbol.MustStartWithCapitalLetterForJSX {
+				slot.needsCapitalForJSX = 1
+			}
+		} else {
+			needsCapitalForJSX := uint32(0)
+			if symbol.MustStartWithCapitalLetterForJSX {
+				needsCapitalForJSX = 1
+			}
+			i = uint32(len(*slots))
+			*slots = append(*slots, symbolSlot{
+				count:              stable.Count,
+				needsCapitalForJSX: needsCapitalForJSX,
+			})
+			r.topLevelSymbolToSlot[stable.Ref] = i
+		}
+	}
 }
 
 func (r *MinifyRenamer) AssignNamesByFrequency(minifier *js_ast.NameMinifier) {
@@ -184,6 +254,7 @@ func (r *MinifyRenamer) AssignNamesByFrequency(minifier *js_ast.NameMinifier) {
 		// Assign names to symbols
 		nextName := 0
 		for _, data := range sorted {
+			slot := &slots[data.slot]
 			name := minifier.NumberToMinifiedName(nextName)
 			nextName++
 
@@ -199,6 +270,14 @@ func (r *MinifyRenamer) AssignNamesByFrequency(minifier *js_ast.NameMinifier) {
 					nextName++
 				}
 
+				// Make sure names of symbols used in JSX elements start with a capital letter
+				if slot.needsCapitalForJSX != 0 {
+					for name[0] >= 'a' && name[0] <= 'z' {
+						name = minifier.NumberToMinifiedName(nextName)
+						nextName++
+					}
+				}
+
 			case js_ast.SlotLabel:
 				for js_lexer.Keywords[name] != 0 {
 					name = minifier.NumberToMinifiedName(nextName)
@@ -211,7 +290,7 @@ func (r *MinifyRenamer) AssignNamesByFrequency(minifier *js_ast.NameMinifier) {
 				name = "#" + name
 			}
 
-			slots[data.slot].name = name
+			slot.name = name
 		}
 	}
 }
@@ -274,8 +353,8 @@ func assignNestedScopeSlotsHelper(scope *js_ast.Scope, symbols []js_ast.Symbol,
 	}
 
 	// Labels are always declared in a nested scope, so we don't need to check.
-	if scope.LabelRef != js_ast.InvalidRef {
-		symbol := &symbols[scope.LabelRef.InnerIndex]
+	if scope.Label.Ref != js_ast.InvalidRef {
+		symbol := &symbols[scope.Label.Ref.InnerIndex]
 		symbol.NestedScopeSlot = ast.MakeIndex32(slot[js_ast.SlotLabel])
 		slot[js_ast.SlotLabel]++
 	}
@@ -303,28 +382,6 @@ func (a slotAndCountArray) Less(i int, j int) bool {
 	return ai.count > aj.count || (ai.count == aj.count && ai.slot < aj.slot)
 }
 
-// The sort order here is arbitrary but needs to be consistent between builds.
-// The InnerIndex should be stable because the parser for a single file is
-// single-threaded and deterministically assigns out InnerIndex values
-// sequentially. But the SourceIndex should be unstable because the main thread
-// assigns out source index values sequentially to newly-discovered dependencies
-// in a multi-threaded producer/consumer relationship. So instead we use the
-// index of the source in the DFS order over all entry points for stability.
-type StableRef struct {
-	StableSourceIndex uint32
-	Ref               js_ast.Ref
-}
-
-// This type is just so we can use Go's native sort function
-type StableRefArray []StableRef
-
-func (a StableRefArray) Len() int          { return len(a) }
-func (a StableRefArray) Swap(i int, j int) { a[i], a[j] = a[j], a[i] }
-func (a StableRefArray) Less(i int, j int) bool {
-	ai, aj := a[i], a[j]
-	return ai.StableSourceIndex < aj.StableSourceIndex || (ai.StableSourceIndex == aj.StableSourceIndex && ai.Ref.InnerIndex < aj.Ref.InnerIndex)
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // NumberRenamer
 
@@ -371,8 +428,16 @@ func (r *NumberRenamer) assignName(scope *numberScope, ref js_ast.Ref) {
 		return
 	}
 
+	// Make sure names of symbols used in JSX elements start with a capital letter
+	originalName := symbol.OriginalName
+	if symbol.MustStartWithCapitalLetterForJSX {
+		if first := rune(originalName[0]); first >= 'a' && first <= 'z' {
+			originalName = fmt.Sprintf("%c%s", first+('A'-'a'), originalName[1:])
+		}
+	}
+
 	// Compute a new name
-	name := scope.findUnusedName(symbol.OriginalName)
+	name := scope.findUnusedName(originalName)
 
 	// Store the new name
 	if inner == nil {
