@@ -13,6 +13,7 @@ import (
 	"github.com/evanw/esbuild/internal/cache"
 	"github.com/evanw/esbuild/internal/config"
 	"github.com/evanw/esbuild/internal/fs"
+	"github.com/evanw/esbuild/internal/helpers"
 	"github.com/evanw/esbuild/internal/js_ast"
 	"github.com/evanw/esbuild/internal/js_lexer"
 	"github.com/evanw/esbuild/internal/js_printer"
@@ -82,6 +83,9 @@ type SideEffectsData struct {
 	Source *logger.Source
 	Range  logger.Range
 
+	// If non-empty, this false value came from a plugin
+	PluginName string
+
 	// If true, "sideEffects" was an array. If false, "sideEffects" was false.
 	IsSideEffectsArrayInJSON bool
 }
@@ -102,6 +106,8 @@ type ResolveResult struct {
 	// effects. This means they should be removed if unused.
 	PrimarySideEffectsData *SideEffectsData
 
+	TSTarget *config.TSTarget
+
 	IsExternal bool
 
 	// If true, the class field transform should use Object.defineProperty().
@@ -123,14 +129,16 @@ type DebugMeta struct {
 }
 
 func (dm DebugMeta) LogErrorMsg(log logger.Log, source *logger.Source, r logger.Range, text string) {
+	tracker := logger.MakeLineColumnTracker(source)
+
 	msg := logger.Msg{
 		Kind:  logger.Error,
-		Data:  logger.RangeData(source, r, text),
+		Data:  logger.RangeData(&tracker, r, text),
 		Notes: dm.notes,
 	}
 
 	if source != nil && dm.suggestionMessage != "" {
-		data := logger.RangeData(source, r, dm.suggestionMessage)
+		data := logger.RangeData(&tracker, r, dm.suggestionMessage)
 		data.Location.Suggestion = dm.suggestionText
 		msg.Notes = append(msg.Notes, data)
 	}
@@ -202,6 +210,7 @@ type resolver struct {
 type resolverQuery struct {
 	*resolver
 	debugLogs *debugLogs
+	kind      ast.ImportKind
 }
 
 func NewResolver(fs fs.FS, log logger.Log, caches *cache.CacheSet, options config.Options) Resolver {
@@ -260,7 +269,10 @@ func NewResolver(fs fs.FS, log logger.Log, caches *cache.CacheSet, options confi
 }
 
 func (rr *resolver) Resolve(sourceDir string, importPath string, kind ast.ImportKind) (*ResolveResult, DebugMeta) {
-	r := resolverQuery{resolver: rr}
+	r := resolverQuery{
+		resolver: rr,
+		kind:     kind,
+	}
 	if r.log.Level <= logger.LevelDebug {
 		r.debugLogs = &debugLogs{what: fmt.Sprintf(
 			"Resolving import %q in directory %q of type %q",
@@ -280,7 +292,10 @@ func (rr *resolver) Resolve(sourceDir string, importPath string, kind ast.Import
 		strings.HasPrefix(importPath, "https://") ||
 
 		// "background: url(//example.com/images/image.png);"
-		strings.HasPrefix(importPath, "//") {
+		strings.HasPrefix(importPath, "//") ||
+
+		// "import fs from 'node:fs'"
+		(r.options.Platform == config.PlatformNode && strings.HasPrefix(importPath, "node:")) {
 
 		if r.debugLogs != nil {
 			r.debugLogs.addNote("Marking this path as implicitly external")
@@ -330,7 +345,7 @@ func (rr *resolver) Resolve(sourceDir string, importPath string, kind ast.Import
 	r.mutex.Lock()
 	defer r.mutex.Unlock()
 
-	result, debug := r.resolveWithoutSymlinks(sourceDir, importPath, kind)
+	result, debug := r.resolveWithoutSymlinks(sourceDir, importPath)
 	if result == nil {
 		// If resolution failed, try again with the URL query and/or hash removed
 		suffix := strings.IndexAny(importPath, "?#")
@@ -341,7 +356,7 @@ func (rr *resolver) Resolve(sourceDir string, importPath string, kind ast.Import
 		if r.debugLogs != nil {
 			r.debugLogs.addNote(fmt.Sprintf("Retrying resolution after removing the suffix %q", importPath[suffix:]))
 		}
-		if result2, debug2 := r.resolveWithoutSymlinks(sourceDir, importPath[:suffix], kind); result2 == nil {
+		if result2, debug2 := r.resolveWithoutSymlinks(sourceDir, importPath[:suffix]); result2 == nil {
 			r.flushDebugLogs(flushDueToFailure)
 			return nil, debug
 		} else {
@@ -388,13 +403,16 @@ func (rr *resolver) ResolveAbs(absPath string) *ResolveResult {
 }
 
 func (rr *resolver) ProbeResolvePackageAsRelative(sourceDir string, importPath string, kind ast.ImportKind) *ResolveResult {
-	r := resolverQuery{resolver: rr}
+	r := resolverQuery{
+		resolver: rr,
+		kind:     kind,
+	}
 	absPath := r.fs.Join(sourceDir, importPath)
 
 	r.mutex.Lock()
 	defer r.mutex.Unlock()
 
-	if pair, ok, diffCase := r.loadAsFileOrDirectory(absPath, kind); ok {
+	if pair, ok, diffCase := r.loadAsFileOrDirectory(absPath); ok {
 		result := &ResolveResult{PathPair: pair, DifferentCase: diffCase}
 		r.finalizeResolve(result)
 		r.flushDebugLogs(flushDueToSuccess)
@@ -442,25 +460,6 @@ func (r resolverQuery) flushDebugLogs(mode flushMode) {
 	}
 }
 
-func IsInsideNodeModules(path string) bool {
-	for {
-		// This is written in a platform-independent manner because it's run on
-		// user-specified paths which can be arbitrary non-file-system things. So
-		// for example Windows paths may end up being used on Unix or URLs may end
-		// up being used on Windows. Be consistently agnostic to which kind of
-		// slash is used on all platforms.
-		slash := strings.LastIndexAny(path, "/\\")
-		if slash == -1 {
-			return false
-		}
-		dir, base := path[:slash], path[slash+1:]
-		if base == "node_modules" {
-			return true
-		}
-		path = dir
-	}
-}
-
 func (r resolverQuery) finalizeResolve(result *ResolveResult) {
 	for _, path := range result.PathPair.iter() {
 		if path.Namespace == "file" {
@@ -506,23 +505,46 @@ func (r resolverQuery) finalizeResolve(result *ResolveResult) {
 
 				// Copy various fields from the nearest enclosing "tsconfig.json" file if present
 				if path == &result.PathPair.Primary && dirInfo.enclosingTSConfigJSON != nil {
-					result.JSXFactory = dirInfo.enclosingTSConfigJSON.JSXFactory
-					result.JSXFragment = dirInfo.enclosingTSConfigJSON.JSXFragmentFactory
-					result.UseDefineForClassFieldsTS = dirInfo.enclosingTSConfigJSON.UseDefineForClassFields
-					result.PreserveUnusedImportsTS = dirInfo.enclosingTSConfigJSON.PreserveImportsNotUsedAsValues
-
-					if r.debugLogs != nil {
-						r.debugLogs.addNote(fmt.Sprintf("This import is under the effect of %q",
-							dirInfo.enclosingTSConfigJSON.AbsPath))
-						if result.JSXFactory != nil {
-							r.debugLogs.addNote(fmt.Sprintf("\"jsxFactory\" is %q due to %q",
-								strings.Join(result.JSXFactory, "."),
-								dirInfo.enclosingTSConfigJSON.AbsPath))
+					// Except don't do this if we're inside a "node_modules" directory. Package
+					// authors often publish their "tsconfig.json" files to npm because of
+					// npm's default-include publishing model and because these authors
+					// probably don't know about ".npmignore" files.
+					//
+					// People trying to use these packages with esbuild have historically
+					// complained that esbuild is respecting "tsconfig.json" in these cases.
+					// The assumption is that the package author published these files by
+					// accident.
+					//
+					// Ignoring "tsconfig.json" files inside "node_modules" directories breaks
+					// the use case of publishing TypeScript code and having it be transpiled
+					// for you, but that's the uncommon case and likely doesn't work with
+					// many other tools anyway. So now these files are ignored.
+					if helpers.IsInsideNodeModules(result.PathPair.Primary.Text) {
+						if r.debugLogs != nil {
+							r.debugLogs.addNote(fmt.Sprintf("Ignoring %q because %q is inside \"node_modules\"",
+								dirInfo.enclosingTSConfigJSON.AbsPath,
+								result.PathPair.Primary.Text))
 						}
-						if result.JSXFragment != nil {
-							r.debugLogs.addNote(fmt.Sprintf("\"jsxFragment\" is %q due to %q",
-								strings.Join(result.JSXFragment, "."),
+					} else {
+						result.JSXFactory = dirInfo.enclosingTSConfigJSON.JSXFactory
+						result.JSXFragment = dirInfo.enclosingTSConfigJSON.JSXFragmentFactory
+						result.UseDefineForClassFieldsTS = dirInfo.enclosingTSConfigJSON.UseDefineForClassFields
+						result.PreserveUnusedImportsTS = dirInfo.enclosingTSConfigJSON.PreserveImportsNotUsedAsValues
+						result.TSTarget = dirInfo.enclosingTSConfigJSON.TSTarget
+
+						if r.debugLogs != nil {
+							r.debugLogs.addNote(fmt.Sprintf("This import is under the effect of %q",
 								dirInfo.enclosingTSConfigJSON.AbsPath))
+							if result.JSXFactory != nil {
+								r.debugLogs.addNote(fmt.Sprintf("\"jsxFactory\" is %q due to %q",
+									strings.Join(result.JSXFactory, "."),
+									dirInfo.enclosingTSConfigJSON.AbsPath))
+							}
+							if result.JSXFragment != nil {
+								r.debugLogs.addNote(fmt.Sprintf("\"jsxFragment\" is %q due to %q",
+									strings.Join(result.JSXFragment, "."),
+									dirInfo.enclosingTSConfigJSON.AbsPath))
+							}
 						}
 					}
 				}
@@ -557,7 +579,7 @@ func (r resolverQuery) finalizeResolve(result *ResolveResult) {
 	}
 }
 
-func (r resolverQuery) resolveWithoutSymlinks(sourceDir string, importPath string, kind ast.ImportKind) (*ResolveResult, DebugMeta) {
+func (r resolverQuery) resolveWithoutSymlinks(sourceDir string, importPath string) (*ResolveResult, DebugMeta) {
 	// This implements the module resolution algorithm from node.js, which is
 	// described here: https://nodejs.org/api/modules.html#modules_all_together
 	var result ResolveResult
@@ -578,7 +600,7 @@ func (r resolverQuery) resolveWithoutSymlinks(sourceDir string, importPath strin
 
 		// First, check path overrides from the nearest enclosing TypeScript "tsconfig.json" file
 		if dirInfo := r.dirInfoCached(sourceDir); dirInfo != nil && dirInfo.enclosingTSConfigJSON != nil && dirInfo.enclosingTSConfigJSON.Paths != nil {
-			if absolute, ok, diffCase := r.matchTSConfigPaths(dirInfo.enclosingTSConfigJSON, importPath, kind); ok {
+			if absolute, ok, diffCase := r.matchTSConfigPaths(dirInfo.enclosingTSConfigJSON, importPath); ok {
 				return &ResolveResult{PathPair: absolute, DifferentCase: diffCase}, DebugMeta{}
 			}
 		}
@@ -595,7 +617,7 @@ func (r resolverQuery) resolveWithoutSymlinks(sourceDir string, importPath strin
 		}
 
 		// Run node's resolution rules (e.g. adding ".js")
-		if absolute, ok, diffCase := r.loadAsFileOrDirectory(importPath, kind); ok {
+		if absolute, ok, diffCase := r.loadAsFileOrDirectory(importPath); ok {
 			return &ResolveResult{PathPair: absolute, DifferentCase: diffCase}, DebugMeta{}
 		}
 		return nil, DebugMeta{}
@@ -604,7 +626,7 @@ func (r resolverQuery) resolveWithoutSymlinks(sourceDir string, importPath strin
 	// Check both relative and package paths for CSS URL tokens, with relative
 	// paths taking precedence over package paths to match Webpack behavior.
 	isPackagePath := IsPackagePath(importPath)
-	checkRelative := !isPackagePath || kind == ast.ImportURL
+	checkRelative := !isPackagePath || r.kind == ast.ImportURL
 	checkPackage := isPackagePath
 
 	if checkRelative {
@@ -624,7 +646,7 @@ func (r resolverQuery) resolveWithoutSymlinks(sourceDir string, importPath strin
 				if remapped == nil {
 					return &ResolveResult{PathPair: PathPair{Primary: logger.Path{Text: absPath, Namespace: "file", Flags: logger.PathDisabled}}}, DebugMeta{}
 				}
-				if remappedResult, ok, diffCase, _ := r.resolveWithoutRemapping(importDirInfo.enclosingBrowserScope, *remapped, kind); ok {
+				if remappedResult, ok, diffCase, _ := r.resolveWithoutRemapping(importDirInfo.enclosingBrowserScope, *remapped); ok {
 					result = ResolveResult{PathPair: remappedResult, DifferentCase: diffCase}
 					checkRelative = false
 					checkPackage = false
@@ -633,7 +655,7 @@ func (r resolverQuery) resolveWithoutSymlinks(sourceDir string, importPath strin
 		}
 
 		if checkRelative {
-			if absolute, ok, diffCase := r.loadAsFileOrDirectory(absPath, kind); ok {
+			if absolute, ok, diffCase := r.loadAsFileOrDirectory(absPath); ok {
 				checkPackage = false
 				result = ResolveResult{PathPair: absolute, DifferentCase: diffCase}
 			} else if !checkPackage {
@@ -674,7 +696,7 @@ func (r resolverQuery) resolveWithoutSymlinks(sourceDir string, importPath strin
 		if remapped, ok := r.checkBrowserMap(sourceDirInfo, importPath, packagePathKind); ok {
 			if remapped == nil {
 				// "browser": {"module": false}
-				if absolute, ok, diffCase, _ := r.loadNodeModules(importPath, kind, sourceDirInfo); ok {
+				if absolute, ok, diffCase, _ := r.loadNodeModules(importPath, sourceDirInfo); ok {
 					absolute.Primary = logger.Path{Text: absolute.Primary.Text, Namespace: "file", Flags: logger.PathDisabled}
 					if absolute.HasSecondary() {
 						absolute.Secondary = logger.Path{Text: absolute.Secondary.Text, Namespace: "file", Flags: logger.PathDisabled}
@@ -691,7 +713,7 @@ func (r resolverQuery) resolveWithoutSymlinks(sourceDir string, importPath strin
 			sourceDirInfo = sourceDirInfo.enclosingBrowserScope
 		}
 
-		if absolute, ok, diffCase, debug := r.resolveWithoutRemapping(sourceDirInfo, importPath, kind); ok {
+		if absolute, ok, diffCase, debug := r.resolveWithoutRemapping(sourceDirInfo, importPath); ok {
 			result = ResolveResult{PathPair: absolute, DifferentCase: diffCase}
 		} else {
 			// Note: node's "self references" are not currently supported
@@ -702,11 +724,11 @@ func (r resolverQuery) resolveWithoutSymlinks(sourceDir string, importPath strin
 	return &result, DebugMeta{}
 }
 
-func (r resolverQuery) resolveWithoutRemapping(sourceDirInfo *dirInfo, importPath string, kind ast.ImportKind) (PathPair, bool, *fs.DifferentCase, DebugMeta) {
+func (r resolverQuery) resolveWithoutRemapping(sourceDirInfo *dirInfo, importPath string) (PathPair, bool, *fs.DifferentCase, DebugMeta) {
 	if IsPackagePath(importPath) {
-		return r.loadNodeModules(importPath, kind, sourceDirInfo)
+		return r.loadNodeModules(importPath, sourceDirInfo)
 	} else {
-		pair, ok, diffCase := r.loadAsFileOrDirectory(r.fs.Join(sourceDirInfo.absPath, importPath), kind)
+		pair, ok, diffCase := r.loadAsFileOrDirectory(r.fs.Join(sourceDirInfo.absPath, importPath))
 		return pair, ok, diffCase, DebugMeta{}
 	}
 }
@@ -818,6 +840,7 @@ func (r resolverQuery) parseTSConfig(file string, visited map[string]bool) (*TSC
 		PrettyPath: r.PrettyPath(keyPath),
 		Contents:   contents,
 	}
+	tracker := logger.MakeLineColumnTracker(&source)
 	fileDir := r.fs.Dir(file)
 
 	result := ParseTSConfigJSON(r.log, source, &r.caches.JSONCache, func(extends string, extendsRange logger.Range) *TSConfigJSON {
@@ -840,10 +863,10 @@ func (r resolverQuery) parseTSConfig(file string, visited map[string]bool) (*TSC
 						} else if err == syscall.ENOENT {
 							continue
 						} else if err == errParseErrorImportCycle {
-							r.log.AddRangeWarning(&source, extendsRange,
+							r.log.AddRangeWarning(&tracker, extendsRange,
 								fmt.Sprintf("Base config file %q forms cycle", extends))
 						} else if err != errParseErrorAlreadyLogged {
-							r.log.AddRangeError(&source, extendsRange,
+							r.log.AddRangeError(&tracker, extendsRange,
 								fmt.Sprintf("Cannot read file %q: %s",
 									r.PrettyPath(logger.Path{Text: fileToCheck, Namespace: "file"}), err.Error()))
 						}
@@ -871,10 +894,10 @@ func (r resolverQuery) parseTSConfig(file string, visited map[string]bool) (*TSC
 				} else if err == syscall.ENOENT {
 					continue
 				} else if err == errParseErrorImportCycle {
-					r.log.AddRangeWarning(&source, extendsRange,
+					r.log.AddRangeWarning(&tracker, extendsRange,
 						fmt.Sprintf("Base config file %q forms cycle", extends))
 				} else if err != errParseErrorAlreadyLogged {
-					r.log.AddRangeError(&source, extendsRange,
+					r.log.AddRangeError(&tracker, extendsRange,
 						fmt.Sprintf("Cannot read file %q: %s",
 							r.PrettyPath(logger.Path{Text: fileToCheck, Namespace: "file"}), err.Error()))
 				}
@@ -883,8 +906,8 @@ func (r resolverQuery) parseTSConfig(file string, visited map[string]bool) (*TSC
 		}
 
 		// Suppress warnings about missing base config files inside "node_modules"
-		if !IsInsideNodeModules(file) {
-			r.log.AddRangeWarning(&source, extendsRange,
+		if !helpers.IsInsideNodeModules(file) {
+			r.log.AddRangeWarning(&tracker, extendsRange,
 				fmt.Sprintf("Cannot find base config file %q", extends))
 		}
 
@@ -1168,7 +1191,7 @@ func getProperty(json js_ast.Expr, name string) (js_ast.Expr, logger.Loc, bool)
 		for _, prop := range obj.Properties {
 			if key, ok := prop.Key.Data.(*js_ast.EString); ok && key.Value != nil &&
 				len(key.Value) == len(name) && js_lexer.UTF16ToString(key.Value) == name {
-				return *prop.Value, prop.Key.Loc, true
+				return prop.ValueOrNil, prop.Key.Loc, true
 			}
 		}
 	}
@@ -1189,10 +1212,10 @@ func getBool(json js_ast.Expr) (bool, bool) {
 	return false, false
 }
 
-func (r resolverQuery) loadAsFileOrDirectory(path string, kind ast.ImportKind) (PathPair, bool, *fs.DifferentCase) {
+func (r resolverQuery) loadAsFileOrDirectory(path string) (PathPair, bool, *fs.DifferentCase) {
 	// Use a special import order for CSS "@import" imports
 	extensionOrder := r.options.ExtensionOrder
-	if kind == ast.ImportAt || kind == ast.ImportAtConditional {
+	if r.kind == ast.ImportAt || r.kind == ast.ImportAtConditional {
 		extensionOrder = r.atImportExtensionOrder
 	}
 
@@ -1310,7 +1333,7 @@ func (r resolverQuery) loadAsFileOrDirectory(path string, kind ast.ImportKind) (
 					// with this same path. The goal of this code is to avoid having
 					// both the "module" file and the "main" file in the bundle at the
 					// same time.
-					if kind != ast.ImportRequire {
+					if r.kind != ast.ImportRequire {
 						if r.debugLogs != nil {
 							r.debugLogs.addNote(fmt.Sprintf("Resolved to %q using the \"module\" field in %q",
 								absolute.Primary.Text, dirInfo.packageJSON.source.KeyPath.Text))
@@ -1349,7 +1372,7 @@ func (r resolverQuery) loadAsFileOrDirectory(path string, kind ast.ImportKind) (
 
 // This closely follows the behavior of "tryLoadModuleUsingPaths()" in the
 // official TypeScript compiler
-func (r resolverQuery) matchTSConfigPaths(tsConfigJSON *TSConfigJSON, path string, kind ast.ImportKind) (PathPair, bool, *fs.DifferentCase) {
+func (r resolverQuery) matchTSConfigPaths(tsConfigJSON *TSConfigJSON, path string) (PathPair, bool, *fs.DifferentCase) {
 	if r.debugLogs != nil {
 		r.debugLogs.addNote(fmt.Sprintf("Matching %q against \"paths\" in %q", path, tsConfigJSON.AbsPath))
 	}
@@ -1379,7 +1402,7 @@ func (r resolverQuery) matchTSConfigPaths(tsConfigJSON *TSConfigJSON, path strin
 				if !r.fs.IsAbs(originalPath) {
 					absoluteOriginalPath = r.fs.Join(absBaseURL, originalPath)
 				}
-				if absolute, ok, diffCase := r.loadAsFileOrDirectory(absoluteOriginalPath, kind); ok {
+				if absolute, ok, diffCase := r.loadAsFileOrDirectory(absoluteOriginalPath); ok {
 					return absolute, true, diffCase
 				}
 			}
@@ -1436,7 +1459,7 @@ func (r resolverQuery) matchTSConfigPaths(tsConfigJSON *TSConfigJSON, path strin
 			if !r.fs.IsAbs(originalPath) {
 				absoluteOriginalPath = r.fs.Join(absBaseURL, originalPath)
 			}
-			if absolute, ok, diffCase := r.loadAsFileOrDirectory(absoluteOriginalPath, kind); ok {
+			if absolute, ok, diffCase := r.loadAsFileOrDirectory(absoluteOriginalPath); ok {
 				return absolute, true, diffCase
 			}
 		}
@@ -1445,7 +1468,7 @@ func (r resolverQuery) matchTSConfigPaths(tsConfigJSON *TSConfigJSON, path strin
 	return PathPair{}, false, nil
 }
 
-func (r resolverQuery) loadNodeModules(importPath string, kind ast.ImportKind, dirInfo *dirInfo) (PathPair, bool, *fs.DifferentCase, DebugMeta) {
+func (r resolverQuery) loadNodeModules(importPath string, dirInfo *dirInfo) (PathPair, bool, *fs.DifferentCase, DebugMeta) {
 	if r.debugLogs != nil {
 		r.debugLogs.addNote(fmt.Sprintf("Searching for %q in \"node_modules\" directories starting from %q", importPath, dirInfo.absPath))
 		r.debugLogs.increaseIndent()
@@ -1456,7 +1479,7 @@ func (r resolverQuery) loadNodeModules(importPath string, kind ast.ImportKind, d
 	if dirInfo.enclosingTSConfigJSON != nil {
 		// Try path substitutions first
 		if dirInfo.enclosingTSConfigJSON.Paths != nil {
-			if absolute, ok, diffCase := r.matchTSConfigPaths(dirInfo.enclosingTSConfigJSON, importPath, kind); ok {
+			if absolute, ok, diffCase := r.matchTSConfigPaths(dirInfo.enclosingTSConfigJSON, importPath); ok {
 				return absolute, true, diffCase, DebugMeta{}
 			}
 		}
@@ -1464,7 +1487,7 @@ func (r resolverQuery) loadNodeModules(importPath string, kind ast.ImportKind, d
 		// Try looking up the path relative to the base URL
 		if dirInfo.enclosingTSConfigJSON.BaseURL != nil {
 			basePath := r.fs.Join(*dirInfo.enclosingTSConfigJSON.BaseURL, importPath)
-			if absolute, ok, diffCase := r.loadAsFileOrDirectory(basePath, kind); ok {
+			if absolute, ok, diffCase := r.loadAsFileOrDirectory(basePath); ok {
 				return absolute, true, diffCase, DebugMeta{}
 			}
 		}
@@ -1499,7 +1522,7 @@ func (r resolverQuery) loadNodeModules(importPath string, kind ast.ImportKind, d
 
 						// The condition set is determined by the kind of import
 						conditions := r.esmConditionsDefault
-						switch kind {
+						switch r.kind {
 						case ast.ImportStmt, ast.ImportDynamic:
 							conditions = r.esmConditionsImport
 						case ast.ImportRequire, ast.ImportRequireResolve:
@@ -1547,7 +1570,7 @@ func (r resolverQuery) loadNodeModules(importPath string, kind ast.ImportKind, d
 								if r.debugLogs != nil {
 									r.debugLogs.addNote(fmt.Sprintf("The resolved path %q is inexact", absResolvedPath))
 								}
-								if absolute, ok, diffCase := r.loadAsFileOrDirectory(absResolvedPath, kind); ok {
+								if absolute, ok, diffCase := r.loadAsFileOrDirectory(absResolvedPath); ok {
 									return absolute, true, diffCase, DebugMeta{}
 								}
 								status = peStatusModuleNotFound
@@ -1560,13 +1583,14 @@ func (r resolverQuery) loadNodeModules(importPath string, kind ast.ImportKind, d
 						}
 
 						// Provide additional details about the failure to help with debugging
+						tracker := logger.MakeLineColumnTracker(&packageJSON.source)
 						switch status {
 						case peStatusInvalidModuleSpecifier:
-							debugMeta.notes = []logger.MsgData{logger.RangeData(&packageJSON.source, debug.token,
+							debugMeta.notes = []logger.MsgData{logger.RangeData(&tracker, debug.token,
 								fmt.Sprintf("The module specifier %q is invalid", resolvedPath))}
 
 						case peStatusInvalidPackageConfiguration:
-							debugMeta.notes = []logger.MsgData{logger.RangeData(&packageJSON.source, debug.token,
+							debugMeta.notes = []logger.MsgData{logger.RangeData(&tracker, debug.token,
 								"The package configuration has an invalid value here")}
 
 						case peStatusInvalidPackageTarget:
@@ -1577,14 +1601,14 @@ func (r resolverQuery) loadNodeModules(importPath string, kind ast.ImportKind, d
 								// configuration error
 								why = "The package configuration has an invalid value here"
 							}
-							debugMeta.notes = []logger.MsgData{logger.RangeData(&packageJSON.source, debug.token, why)}
+							debugMeta.notes = []logger.MsgData{logger.RangeData(&tracker, debug.token, why)}
 
 						case peStatusPackagePathNotExported:
-							debugMeta.notes = []logger.MsgData{logger.RangeData(&packageJSON.source, debug.token,
+							debugMeta.notes = []logger.MsgData{logger.RangeData(&tracker, debug.token,
 								fmt.Sprintf("The path %q is not exported by package %q", esmPackageSubpath, esmPackageName))}
 
 							// If this fails, try to resolve it using the old algorithm
-							if absolute, ok, _ := r.loadAsFileOrDirectory(absPath, kind); ok && absolute.Primary.Namespace == "file" {
+							if absolute, ok, _ := r.loadAsFileOrDirectory(absPath); ok && absolute.Primary.Namespace == "file" {
 								if relPath, ok := r.fs.Rel(absPkgPath, absolute.Primary.Text); ok {
 									query := "." + path.Join("/", strings.ReplaceAll(relPath, "\\", "/"))
 
@@ -1592,7 +1616,7 @@ func (r resolverQuery) loadNodeModules(importPath string, kind ast.ImportKind, d
 									// "exports" map for the currently-active set of conditions
 									if ok, subpath, token := r.esmPackageExportsReverseResolve(
 										query, pkgDirInfo.packageJSON.exportsMap.root, conditions); ok {
-										debugMeta.notes = append(debugMeta.notes, logger.RangeData(&pkgDirInfo.packageJSON.source, token,
+										debugMeta.notes = append(debugMeta.notes, logger.RangeData(&tracker, token,
 											fmt.Sprintf("The file %q is exported at path %q", query, subpath)))
 
 										// Provide an inline suggestion message with the correct import path
@@ -1605,11 +1629,11 @@ func (r resolverQuery) loadNodeModules(importPath string, kind ast.ImportKind, d
 							}
 
 						case peStatusModuleNotFound:
-							debugMeta.notes = []logger.MsgData{logger.RangeData(&packageJSON.source, debug.token,
+							debugMeta.notes = []logger.MsgData{logger.RangeData(&tracker, debug.token,
 								fmt.Sprintf("The module %q was not found on the file system", resolvedPath))}
 
 						case peStatusUnsupportedDirectoryImport:
-							debugMeta.notes = []logger.MsgData{logger.RangeData(&packageJSON.source, debug.token,
+							debugMeta.notes = []logger.MsgData{logger.RangeData(&tracker, debug.token,
 								fmt.Sprintf("Importing the directory %q is not supported", resolvedPath))}
 
 						case peStatusUndefinedNoConditionsMatch:
@@ -1626,18 +1650,18 @@ func (r resolverQuery) loadNodeModules(importPath string, kind ast.ImportKind, d
 							}
 							sort.Strings(keys)
 							debugMeta.notes = []logger.MsgData{
-								logger.RangeData(&packageJSON.source, packageJSON.exportsMap.root.firstToken,
+								logger.RangeData(&tracker, packageJSON.exportsMap.root.firstToken,
 									fmt.Sprintf("The path %q is not currently exported by package %q",
 										esmPackageSubpath, esmPackageName)),
-								logger.RangeData(&packageJSON.source, debug.token,
+								logger.RangeData(&tracker, debug.token,
 									fmt.Sprintf("None of the conditions provided (%s) match any of the currently active conditions (%s)",
 										prettyPrintConditions(debug.unmatchedConditions),
 										prettyPrintConditions(keys),
 									))}
 							for _, key := range debug.unmatchedConditions {
-								if key == "import" && (kind == ast.ImportRequire || kind == ast.ImportRequireResolve) {
+								if key == "import" && (r.kind == ast.ImportRequire || r.kind == ast.ImportRequireResolve) {
 									debugMeta.suggestionMessage = "Consider using an \"import\" statement to import this file"
-								} else if key == "require" && (kind == ast.ImportStmt || kind == ast.ImportDynamic) {
+								} else if key == "require" && (r.kind == ast.ImportStmt || r.kind == ast.ImportDynamic) {
 									debugMeta.suggestionMessage = "Consider using a \"require()\" call to import this file"
 								}
 							}
@@ -1651,14 +1675,14 @@ func (r resolverQuery) loadNodeModules(importPath string, kind ast.ImportKind, d
 						if remapped == nil {
 							return PathPair{Primary: logger.Path{Text: absPath, Namespace: "file", Flags: logger.PathDisabled}}, true, nil, DebugMeta{}
 						}
-						if remappedResult, ok, diffCase, notes := r.resolveWithoutRemapping(pkgDirInfo.enclosingBrowserScope, *remapped, kind); ok {
+						if remappedResult, ok, diffCase, notes := r.resolveWithoutRemapping(pkgDirInfo.enclosingBrowserScope, *remapped); ok {
 							return remappedResult, true, diffCase, notes
 						}
 					}
 				}
 			}
 
-			if absolute, ok, diffCase := r.loadAsFileOrDirectory(absPath, kind); ok {
+			if absolute, ok, diffCase := r.loadAsFileOrDirectory(absPath); ok {
 				return absolute, true, diffCase, DebugMeta{}
 			}
 		}
@@ -1679,7 +1703,7 @@ func (r resolverQuery) loadNodeModules(importPath string, kind ast.ImportKind, d
 	// published algorithm. See also: https://github.com/nodejs/node/issues/38128.
 	for _, absDir := range r.options.AbsNodePaths {
 		absPath := r.fs.Join(absDir, importPath)
-		if absolute, ok, diffCase := r.loadAsFileOrDirectory(absPath, kind); ok {
+		if absolute, ok, diffCase := r.loadAsFileOrDirectory(absPath); ok {
 			return absolute, true, diffCase, DebugMeta{}
 		}
 	}
@@ -1695,44 +1719,60 @@ func IsPackagePath(path string) bool {
 }
 
 var BuiltInNodeModules = map[string]bool{
-	"assert":         true,
-	"async_hooks":    true,
-	"buffer":         true,
-	"child_process":  true,
-	"cluster":        true,
-	"console":        true,
-	"constants":      true,
-	"crypto":         true,
-	"dgram":          true,
-	"dns":            true,
-	"domain":         true,
-	"events":         true,
-	"fs":             true,
-	"http":           true,
-	"http2":          true,
-	"https":          true,
-	"inspector":      true,
-	"module":         true,
-	"net":            true,
-	"os":             true,
-	"path":           true,
-	"perf_hooks":     true,
-	"process":        true,
-	"punycode":       true,
-	"querystring":    true,
-	"readline":       true,
-	"repl":           true,
-	"stream":         true,
-	"string_decoder": true,
-	"sys":            true,
-	"timers":         true,
-	"tls":            true,
-	"trace_events":   true,
-	"tty":            true,
-	"url":            true,
-	"util":           true,
-	"v8":             true,
-	"vm":             true,
-	"worker_threads": true,
-	"zlib":           true,
+	"_http_agent":         true,
+	"_http_client":        true,
+	"_http_common":        true,
+	"_http_incoming":      true,
+	"_http_outgoing":      true,
+	"_http_server":        true,
+	"_stream_duplex":      true,
+	"_stream_passthrough": true,
+	"_stream_readable":    true,
+	"_stream_transform":   true,
+	"_stream_wrap":        true,
+	"_stream_writable":    true,
+	"_tls_common":         true,
+	"_tls_wrap":           true,
+	"assert":              true,
+	"async_hooks":         true,
+	"buffer":              true,
+	"child_process":       true,
+	"cluster":             true,
+	"console":             true,
+	"constants":           true,
+	"crypto":              true,
+	"dgram":               true,
+	"diagnostics_channel": true,
+	"dns":                 true,
+	"domain":              true,
+	"events":              true,
+	"fs":                  true,
+	"http":                true,
+	"http2":               true,
+	"https":               true,
+	"inspector":           true,
+	"module":              true,
+	"net":                 true,
+	"os":                  true,
+	"path":                true,
+	"perf_hooks":          true,
+	"process":             true,
+	"punycode":            true,
+	"querystring":         true,
+	"readline":            true,
+	"repl":                true,
+	"stream":              true,
+	"string_decoder":      true,
+	"sys":                 true,
+	"timers":              true,
+	"tls":                 true,
+	"trace_events":        true,
+	"tty":                 true,
+	"url":                 true,
+	"util":                true,
+	"v8":                  true,
+	"vm":                  true,
+	"wasi":                true,
+	"worker_threads":      true,
+	"zlib":                true,
 }
