@@ -177,12 +177,34 @@ let ensureServiceIsRunning = (): Promise<Service> => {
       })
 
       stopService = () => {
+        // Close all resources related to the subprocess.
+        child.stdin.close()
+        child.stdout.close()
         child.close()
+        initializeWasCalled = false;
+        longLivedService = undefined;
+        stopService = undefined
+      }
+
+      let writeQueue: Uint8Array[] = []
+      let isQueueLocked = false
+
+      // We need to keep calling "write()" until it actually writes the data
+      const startWriteFromQueueWorker = () => {
+        if (isQueueLocked || writeQueue.length === 0) return
+        isQueueLocked = true
+        child.stdin.write(writeQueue[0]).then(bytesWritten => {
+          isQueueLocked = false
+          if (bytesWritten === writeQueue[0].length) writeQueue.shift()
+          else writeQueue[0] = writeQueue[0].subarray(bytesWritten)
+          startWriteFromQueueWorker()
+        })
       }
 
       const { readFromStdout, afterClose, service } = common.createChannel({
         writeToStdin(bytes) {
-          child.stdin.write(bytes)
+          writeQueue.push(bytes)
+          startWriteFromQueueWorker()
         },
         isSync: false,
         isBrowser: false,
@@ -196,6 +218,13 @@ let ensureServiceIsRunning = (): Promise<Service> => {
           readFromStdout(stdoutBuffer.subarray(0, n))
           readMoreStdout()
         }
+      }).catch(e => {
+        if (e instanceof Deno.errors.Interrupted || e instanceof Deno.errors.BadResource) {
+          // ignore the error if read was interrupted (stdout was closed)
+          afterClose()
+        } else {
+          throw e;
+        }
       })
       readMoreStdout()
 
