@@ -390,6 +390,23 @@ func parseOptionsImpl(
 		case strings.HasPrefix(arg, "--inject:") && buildOpts != nil:
 			buildOpts.Inject = append(buildOpts.Inject, arg[len("--inject:"):])
 
+		case strings.HasPrefix(arg, "--jsx="):
+			value := arg[len("--jsx="):]
+			var mode api.JSXMode
+			switch value {
+			case "transform":
+				mode = api.JSXModeTransform
+			case "preserve":
+				mode = api.JSXModePreserve
+			default:
+				return fmt.Errorf("Invalid jsx: %q (valid: transform, preserve)", value), nil
+			}
+			if buildOpts != nil {
+				buildOpts.JSXMode = mode
+			} else {
+				transformOpts.JSXMode = mode
+			}
+
 		case strings.HasPrefix(arg, "--jsx-factory="):
 			value := arg[len("--jsx-factory="):]
 			if buildOpts != nil {
@@ -527,6 +544,7 @@ func parseTargets(targets []string) (target api.Target, engines []api.Engine, er
 		"es2018": api.ES2018,
 		"es2019": api.ES2019,
 		"es2020": api.ES2020,
+		"es2021": api.ES2021,
 	}
 
 	validEngines := map[string]api.EngineName{
@@ -626,10 +644,10 @@ func runImpl(osArgs []string) int {
 
 	switch {
 	case buildOptions != nil:
-		// Read the "NODE_PATH" from the environment. This is part of node's
-		// module resolution algorithm. Documentation for this can be found here:
-		// https://nodejs.org/api/modules.html#modules_loading_from_the_global_folders
 		for _, key := range os.Environ() {
+			// Read the "NODE_PATH" from the environment. This is part of node's
+			// module resolution algorithm. Documentation for this can be found here:
+			// https://nodejs.org/api/modules.html#modules_loading_from_the_global_folders
 			if strings.HasPrefix(key, "NODE_PATH=") {
 				value := key[len("NODE_PATH="):]
 				separator := ":"
@@ -640,6 +658,12 @@ func runImpl(osArgs []string) int {
 				buildOptions.NodePaths = strings.Split(value, separator)
 				break
 			}
+
+			// Read "NO_COLOR" from the environment. This is a convention that some
+			// software follows. See https://no-color.org/ for more information.
+			if buildOptions.Color == api.ColorIfTerminal && strings.HasPrefix(key, "NO_COLOR=") {
+				buildOptions.Color = api.ColorNever
+			}
 		}
 
 		// Read from stdin when there are no entry points
@@ -669,15 +693,18 @@ func runImpl(osArgs []string) int {
 		// Validate the metafile absolute path and directory ahead of time so we
 		// don't write any output files if it's incorrect. That makes this API
 		// option consistent with how we handle all other API options.
-		var metafileAbsPath string
-		var metafileAbsDir string
+		var writeMetafile func(string)
 		if metafile != nil {
+			var metafileAbsPath string
+			var metafileAbsDir string
+
 			if buildOptions.Outfile == "" && buildOptions.Outdir == "" {
 				// Cannot use "metafile" when writing to stdout
 				logger.PrintErrorToStderr(osArgs, "Cannot use \"metafile\" without an output path")
 				return 1
 			}
-			if realFS, err := fs.RealFS(fs.RealFSOptions{AbsWorkingDir: buildOptions.AbsWorkingDir}); err == nil {
+			realFS, realFSErr := fs.RealFS(fs.RealFSOptions{AbsWorkingDir: buildOptions.AbsWorkingDir})
+			if realFSErr == nil {
 				absPath, ok := realFS.Abs(*metafile)
 				if !ok {
 					logger.PrintErrorToStderr(osArgs, fmt.Sprintf("Invalid metafile path: %s", *metafile))
@@ -688,11 +715,44 @@ func runImpl(osArgs []string) int {
 			} else {
 				// Don't fail in this case since the error will be reported by "api.Build"
 			}
+
+			writeMetafile = func(json string) {
+				if json == "" || realFSErr != nil {
+					return // Don't write out the metafile on build errors
+				}
+				if err != nil {
+					// This should already have been checked above
+					panic(err.Error())
+				}
+				fs.BeforeFileOpen()
+				defer fs.AfterFileClose()
+				if err := fs.MkdirAll(realFS, metafileAbsDir, 0755); err != nil {
+					logger.PrintErrorToStderr(osArgs, fmt.Sprintf(
+						"Failed to create output directory: %s", err.Error()))
+				} else {
+					if err := ioutil.WriteFile(metafileAbsPath, []byte(json), 0644); err != nil {
+						logger.PrintErrorToStderr(osArgs, fmt.Sprintf(
+							"Failed to write to output file: %s", err.Error()))
+					}
+				}
+			}
+
+			// Write out the metafile whenever we rebuild
+			if buildOptions.Watch != nil {
+				buildOptions.Watch.OnRebuild = func(result api.BuildResult) {
+					writeMetafile(result.Metafile)
+				}
+			}
 		}
 
 		// Run the build
 		result := api.Build(*buildOptions)
 
+		// Write the metafile to the file system
+		if writeMetafile != nil {
+			writeMetafile(result.Metafile)
+		}
+
 		// Do not exit if we're in watch mode
 		if buildOptions.Watch != nil {
 			<-make(chan bool)
@@ -703,25 +763,6 @@ func runImpl(osArgs []string) int {
 			return 1
 		}
 
-		// Write the metafile to the file system
-		if metafile != nil {
-			if err != nil {
-				// This should already have been checked above
-				panic(err.Error())
-			}
-			fs.BeforeFileOpen()
-			defer fs.AfterFileClose()
-			if err := os.MkdirAll(metafileAbsDir, 0755); err != nil {
-				logger.PrintErrorToStderr(osArgs, fmt.Sprintf(
-					"Failed to create output directory: %s", err.Error()))
-			} else {
-				if err := ioutil.WriteFile(metafileAbsPath, []byte(result.Metafile), 0644); err != nil {
-					logger.PrintErrorToStderr(osArgs, fmt.Sprintf(
-						"Failed to write to output file: %s", err.Error()))
-				}
-			}
-		}
-
 	case transformOptions != nil:
 		// Read the input from stdin
 		bytes, err := ioutil.ReadAll(os.Stdin)
